/**************************************************************************
*  Copyright (c) 1997-1998 by Joseph Parrello. All Rights Reserved.       *
*  Sites  : http://www.geocities.com/SiliconValley/Hills/8145             *
*           http://users.iol.it/joparrello                                *
*  E-Mail : joparrello@geocities.com                                      *
*                                                                         *
*  Revision:   2.1  (December 15, 1998).                                  *
*                                                                         *
*  TButtonApp application.                                                *
*                                                                         *
*  Read BUTTONAP.TXT for use and license.                                 *
*                                                                         *
*  Only for OWL 5.02 and 5.01 users.                                      *
*  OWL and ObjectWindows are trademarks of Borland International.         *
*                                                                         *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF     *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO    *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A         *
*  PARTICULAR PURPOSE.                                                    *
**************************************************************************/
#include <owl/pch.h>

#include <owl/buttonga.h>
#include <owl/docmanag.h>
#include <owl/filedoc.h>
#include <owl/docking.h>
#include <owl/controlg.h>
#include <owl/propsht.h>
#include <owl/tabctrl.h>

#include <classlib/cmdline.h>

#include "buttonap.h"
#include "buttonmd.h"
#include "buttonm1.h"
#include "buttonev.h"
#include "jpcstbar.h"
#include "jpflmenu.h"
#include "jpfltgrp.h"
#include "jpcontga.h"
#include "buttonad.h"                        // Definition of about dialog.

const char HelpFileName[] = "BUTTON.hlp";

/*****    KEYS-NAMES ADDED TO SUPPORT REGISTRY     ******/
const char OptionsKey[] = "Options";    // where we load/save data for main window.
const char LocationKey[] = "Location";  // The location of each toolbar.
const char ToolbarKey[] = "Toolbar";    // Prefix for each toolbar.
const char ToolbarOptions[] = "ToolbarOptions"; //where we load/save data for toolbars' options.
const char ColorPickerKey[] = "ColorPicker"; //where we load/save data for color picker.
const char PosXKey[] = "PosX";          // The horizontal starting position of toolbar.
const char PosYKey[] = "PosY";          // The vertical starting position of toolbar.
const char WidthKey[] = "Width";        // The width of toolbar.
const char HeightKey[] = "Height";      // The height of toolbar.
const char VisibleKey[] = "Visible";    // The toolbar is visible.
const char ButtonKey[] = "Button";      // Prefix for each button of the toolbar.
const char MWFlagsKey[] = "MainWindowFlags";      // Main Window Flags.
const char MWShowKey[] = "MainWindowShowState";      // Main Window Show State.
const char MWMinPosXKey[] = "MainWindowMinPosX";      // Main Window Min Horizontal Position.
const char MWMinPosYKey[] = "MainWindowMinPosY";      // Main Window Min Vertical Position.
const char MWMaxPosXKey[] = "MainWindowMaxPosX";      // Main Window Max Horizontal Position.
const char MWMaxPosYKey[] = "MainWindowMaxPosY";      // Main Window Max Vertical Position.
const char MWNormPosLeftKey[] = "MainWindowNormalPosLeft";     // Main Window Normal Left Position.
const char MWNormPosTopKey[] = "MainWindowNormalPosTop";       // Main Window Normal Top Position.
const char MWNormPosRightKey[] = "MainWindowNormalPosRight";   // Main Window Normal Right Position.
const char MWNormPosBottomKey[] = "MainWindowNormalPosBottom"; // Main Window Normal Bottom Position.
const char TOOptionalParams[] = "OptionalParams";   // Optional params for all toolbars
const char TODisplayType[] = "DisplayType";         // Text displaying type for all toolbars
const char TOTextPos[] = "TextPosition";            // Text position for all toolbars
const char TOToolbarRows[] = "ToolbarRows";         // Number of rows for all toolbars
const char TOMaxChars[] = "MaxChars";               // Max number of chars for all toolbars
const char TOHotTrack[] = "HotTracking";            // Hot-tracking flag for all toolbars
const char TODivEol[] = "DividerEndOfLine";         // Divider EOL flag for all toolbars
const char TODivVertBar[] = "NoDividerVerticalBar"; // Divider VertBar flag for all toolbars
const char TODivRefGadget[] = "DividerReferenceGadget"; // Divider ref gadget for all toolbars
const char TODockingGrip[] = "DockingGrip";             // Docking grip for all toolbars
const char CPRedValue[] = "RedValue";               // Red value for color picker
const char CPGreenValue[] = "GreenValue";           // Green value for color picker
const char CPBlueValue[] = "BlueValue";             // Blue value for color picker
const char CPFlagValue[] = "FlagValue";             // Flag value for color picker

/********************************************************/
/*          ADDED TO SUPPORT MULTIPLE TOOLBARS          */
/********************************************************/
typedef struct tagTOOLBARDATA
{
    int  GadgetId;
    int  ToolbarPos;
} TOOLBARDATA;
typedef TOOLBARDATA NEAR* NPTOOLBARDATA;
typedef TOOLBARDATA FAR*  LPTOOLBARDATA;

/****     DEFAULT TOOLBAR LOCATION (LOADED WHEN THERE ARE NO DATA FROM REGISTRY    *****/
const TAbsLocation TBarsLocation[] = { alTop, alTop, alTop };

/****     DEFAULT TOOLBAR DATA  (LOADED WHEN THERE ARE NO DATA FROM REGISTRY    *****/
const TOOLBARDATA GadgetList[] =   { //   Button Id,       Toolbar Index (stored in ToolbarList).
                                   { CM_MDIFILENEW,        0 },
                                   { CM_MDIFILEOPEN,       0 },
                                   { CM_FILESAVE,          0 },
                                   { 0,                    0 },
                                   { CM_EDITCUT,           0 },
                                   { CM_EDITCOPY,          0 },
                                   { CM_EDITPASTE,         0 },
                                   { 0,                    0 },
                                   { CM_EDITUNDO,          0 },
                                   { 0,                    0 },
                                   { CM_EDITFIND,          0 },
                                   { CM_EDITFINDNEXT,      0 },
                                   { 0,                    0 },
                                   { CM_FILEPRINT,         0 },
                                   { CM_FILEPRINTPREVIEW,  0 },
                                   { 0,                    0 },
                                   { CM_HELPCONTENTS,      0 },
                                   { 0,                    0 },
                                   { ID_COMBOBOX,          0 },
                                   { CM_TOOLBARCUSTOMIZE,  1 },
                                   { 0,                    1 },
                                   { CM_TOOLBARBITMAP,     1 },
                                   { CM_TOOLBARTEXT,       1 },
                                   { CM_TOOLBARBITMAPTEXT, 1 },
                                   { ID_BUTTONMENU,        2 },
                                   { 0,                    2 },
                                   { ID_COLORPICKER,       2 }
                                   };

//{{TBUTTONApp Implementation}}



//{{DOC_VIEW}}
DEFINE_DOC_TEMPLATE_CLASS(TFileDocument, TBUTTONEditView, DocType1);
//{{DOC_VIEW_END}}

//{{DOC_MANAGER}}
DocType1 __dvt1("All Files", "*.*", 0, "txt", dtAutoDelete | dtUpdateDir | dtOverwritePrompt);
//{{DOC_MANAGER_END}}


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE2(TBUTTONApp, JPMRURegistry, TApplication)
//{{TBUTTONAppRSP_TBL_BEGIN}}
  EV_OWLVIEW(dnCreate, EvNewView),
  EV_OWLVIEW(dnClose,  EvCloseView),
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
  EV_COMMAND(CM_HELPCONTENTS, CmHelpContents),
  EV_COMMAND(CM_HELPUSING, CmHelpUsing),
  EV_WM_DROPFILES,
  EV_WM_WININICHANGE,
  EV_OWLDOCUMENT(dnCreate, EvOwlDocument),
  EV_OWLDOCUMENT(dnRename, EvOwlDocument),
  EV_REGISTERED(MruFileMessage, CmFileSelected),
/*******    ADDED TO SUPPORT FLAT COLOR PICKER NOTIFICATION MESSAGE    ********/
  EV_REGISTERED(JPColorPickerMessage, CmColorPickerSelected),
/*******    ADDED TO SUPPORT FLAT BUTTON MENU NOTIFICATION MESSAGE    ********/
//  EV_REGISTERED(FlatButtonMenuMessage, CmButtonMenuSelected),
/*******    ADDED TO SUPPORT MULTIPLE TOOLBARS            */
  EV_COMMAND_ENABLE(CM_TOOLBARBITMAP, CeToolbarbitmap),
  EV_COMMAND_ENABLE(CM_TOOLBARBITMAPTEXT, CeToolbarbitmaptext),
  EV_COMMAND_ENABLE(CM_TOOLBARBOTTOM, CeToolbarbottom),
  EV_COMMAND_ENABLE(CM_TOOLBARLEFT, CeToolbarleft),
  EV_COMMAND_ENABLE(CM_TOOLBARRIGHT, CeToolbarright),
  EV_COMMAND_ENABLE(CM_TOOLBARTEXT, CeToolbartext),
  EV_COMMAND_ENABLE(CM_TOOLBARTOP, CeToolbartop),
  EV_COMMAND_ENABLE(CM_ONETEXTROW, CeOneTextRow),
  EV_COMMAND_ENABLE(CM_TWOTEXTROW, CeTwoTextRow),
  EV_COMMAND(CM_TOOLBARBITMAP, CmToolbarbitmap),
  EV_COMMAND(CM_TOOLBARBITMAPTEXT, CmToolbarbitmaptext),
  EV_COMMAND(CM_TOOLBARBOTTOM, CmToolbarbottom),
  EV_COMMAND(CM_TOOLBARLEFT, CmToolbarleft),
  EV_COMMAND(CM_TOOLBARRIGHT, CmToolbarright),
  EV_COMMAND(CM_TOOLBARTEXT, CmToolbartext),
  EV_COMMAND(CM_TOOLBARTOP, CmToolbartop),
  EV_COMMAND(CM_ONETEXTROW, CmOneTextRow),
  EV_COMMAND(CM_TWOTEXTROW, CmTwoTextRow),
  EV_COMMAND(CM_DECREASECHARS, CmDecreasechars),
  EV_COMMAND_ENABLE(CM_DECREASECHARS, CeDecreasechars),
  EV_COMMAND(CM_DEFAULTCHARS, CmDefaultchars),
  EV_COMMAND_ENABLE(CM_DEFAULTCHARS, CeDefaultchars),
  EV_COMMAND(CM_INCREASECHARS, CmIncreasechars),
  EV_COMMAND_ENABLE(CM_INCREASECHARS, CeIncreasechars),
  EV_COMMAND(CM_HOTTRACKING, CmHottracking),
  EV_COMMAND_ENABLE(CM_HOTTRACKING, CeHottracking),
  EV_COMMAND(CM_DIVBUTTONS, CmDivbuttons),
  EV_COMMAND_ENABLE(CM_DIVBUTTONS, CeDivbuttons),
  EV_COMMAND(CM_DIVCOMBO, CmDivcombo),
  EV_COMMAND_ENABLE(CM_DIVCOMBO, CeDivcombo),
  EV_COMMAND(CM_DIVNOTHING, CmDivnothing),
  EV_COMMAND_ENABLE(CM_DIVNOTHING, CeDivnothing),
  EV_COMMAND(CM_DIVENDOFLINE, CmDivendofline),
  EV_COMMAND_ENABLE(CM_DIVENDOFLINE, CeDivendofline),
  EV_COMMAND(CM_DIVVERTBAR, CmDivvertbar),
  EV_COMMAND_ENABLE(CM_DIVVERTBAR, CeDivvertbar),
  EV_COMMAND(CM_TOOLBARCUSTOMIZE, CmToolbarcustomize),
  EV_COMMAND_ENABLE(CM_TOOLBARCUSTOMIZE, CeToolbarcustomize),
  EV_COMMAND_ENABLE(ID_BUTTONMENU, CeButtonMenu),
  EV_COMMAND_ENABLE(ID_COLORPICKER, CeColorPicker),
  EV_COMMAND(CM_IEGRAY, CmIEGray),
  EV_COMMAND_ENABLE(CM_IEGRAY, CeIEGray),
  EV_COMMAND(CM_GRAYSCALE, CmGrayscale),
  EV_COMMAND_ENABLE(CM_GRAYSCALE, CeGrayscale),
  EV_COMMAND(CM_LARGEBUTTONS, CmLargebuttons),
  EV_COMMAND_ENABLE(CM_LARGEBUTTONS, CeLargebuttons),
  EV_COMMAND(CM_FITTEXTWIDTH, CmFittextwidth),
  EV_COMMAND_ENABLE(CM_FITTEXTWIDTH, CeFittextwidth),
  EV_COMMAND(CM_DOCKINGGRIP, CmDockinggrip),
  EV_COMMAND_ENABLE(CM_DOCKINGGRIP, CeDockinggrip),
  EV_COMMAND_ENABLE(CM_TOGGLECOMBOBOX, CeTogglecombobox),
  EV_COMMAND(CM_TOGGLECOMBOBOX, CmTogglecombobox),
  EV_COMMAND(CM_FULLOFFICESUPPORT, CmFullofficesupport),
  EV_COMMAND_ENABLE(CM_FULLOFFICESUPPORT, CeFullofficesupport),
//{{TBUTTONAppRSP_TBL_END}}
END_RESPONSE_TABLE;

#if defined(BI_PLAT_WIN16)
     const char* ButtonAppRegKey = "JoParrello\\ButtonApp16\\2.0";
#else
     const char* ButtonAppRegKey = "JoParrello\\ButtonApp32\\2.0";
#endif

//--------------------------------------------------------
// TBUTTONApp
// ~~~~~

// Don't delete JPMRURegistry class as it provides functions for Toolbars' customization.
TBUTTONApp::TBUTTONApp() : TApplication("Button Application 2.1"),
               JPMRURegistry(ButtonAppRegKey, 10, true, false, true, true, false, 30)
, ToolbarList(5, 0, 5)   /******      TOOLBAR LIST ARRAY        *****/
{
  HelpState = false;
  ContextHelp = false;
  HelpCursor = 0;

  Printer = 0;
  Printing = 0;

  SetDocManager(new TDocManager(dmMDI, this));

  // INSERT>> Your constructor code here.
}


TBUTTONApp::~TBUTTONApp()
{
  delete Printer;
  // INSERT>> Your destructor code here.

/*********   DELETE ALL TOOLBARS FROM TOOLBARLIST      ******/
/**   Note : don't remove these lines, you can cause a GPF error!!     ***/
  while (ToolbarList.GetItemsInContainer() > 0)
      ToolbarList.RemoveEntry(0);

  delete GadgetAvailableList;
  if (tbarmenu)
     delete tbarmenu;
}


bool TBUTTONApp::CanClose()
{
  bool result = TApplication::CanClose();

/*****       STORE TOOLBARS DATA AND MAIN WINDOW'S DATA ON THE REGISTRY     ******/
  if (result)
    {
    StoreMainWindowInfoOnRegistry();
    StoreToolbarDataInProfile();
    StoreColorPickerDataOnRegistry();  // Store the color picker data too!!!
    }

  if (result && HelpState)
    GetMainWindow()->WinHelp(HelpFileName, HELP_QUIT, 0);

  return result;
}

void TBUTTONApp::SetupSpeedBar(JPDecMDIFrame* frame, TGadgetWindow::TTileDirection direction, TFont* font)
{
#pragma warn -eas
/********    SET THE INTERNAL VARIABLES TO USE WITH OUR NEW FUNCTIONS          ********/
  toolbarmenu = NULL;
  tbarmenu = NULL;
  divGadget = 0;

  // Retrieve the toolbars' options from the registry.
  HKEY tbarKey;
  tbarKey = CreateRegistrySubkey(GetRegistryRootKey(), ToolbarOptions);
  moreParams  = GetRegistryIntegerValue(tbarKey, TOOptionalParams, JPFlatButtonTextGadget::IEGrayButtons);
  dispType    = GetRegistryIntegerValue(tbarKey, TODisplayType, JPFlatButtonTextGadget::BitmapText);
  textPos     = GetRegistryIntegerValue(tbarKey, TOTextPos, JPFlatButtonTextGadget::Bottom);
  ToolbarRows = (uint) GetRegistryIntegerValue(tbarKey, TOToolbarRows, 1);
  MaxChars    = (uint) GetRegistryIntegerValue(tbarKey, TOMaxChars, 4);
  HotTrack    = (bool) GetRegistryIntegerValue(tbarKey, TOHotTrack, 1);
  divEOL      = (bool) GetRegistryIntegerValue(tbarKey, TODivEol, 0);
  divVertBar  = (bool) GetRegistryIntegerValue(tbarKey, TODivVertBar, 0);
  DockingGrip = (bool) GetRegistryIntegerValue(tbarKey, TODockingGrip, 1);
  int gadgid  = GetRegistryIntegerValue(tbarKey, TODivRefGadget, 0);

  ApxHarbor = new THarbor(*frame);

  /*      STEP 1 :  CREATE YOUR OWN TOOLBARS             */
  /*      Note   :  Hide each toolbar, they will be showed by our functions.         */
  /*      Note   :  if you want to put a font param on each toolbar, create a different instance
                    of it.   If you put the same instance, you will cause a GPF error!!!       */
  cb = new TDockableControlBar(frame, direction, font);
  cb->ShowWindow(SW_HIDE);
  TFont* customFont = new TFont(*font);
  custombar = new TDockableControlBar(frame, direction, customFont);
  custombar->ShowWindow(SW_HIDE);
  TFont* thirdFont = new TFont(*font);
  TDockableControlBar* mydock = new TDockableControlBar(frame, direction, thirdFont);
  mydock->ShowWindow(SW_HIDE);
  mydock->SetCaption("Third Toolbar");
  mydock->SetHintMode(TGadgetWindow::EnterHints);

  /*      STEP 2 :  FILL THE AVAILABLE LIST WITH YOUR GADGETS                                 */
  /*      Note   :  Don't insert separator gadgets. They will be inserted by our functions.   */
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_MDIFILENEW, CM_MDIFILENEW));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_MDIFILEOPEN, CM_MDIFILEOPEN));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_FILESAVE, CM_FILESAVE));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_EDITCUT, CM_EDITCUT));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_EDITCOPY, CM_EDITCOPY));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_EDITPASTE, CM_EDITPASTE));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_EDITUNDO, CM_EDITUNDO));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_EDITFIND, CM_EDITFIND));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_EDITFINDNEXT, CM_EDITFINDNEXT));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_FILEPRINT, CM_FILEPRINT));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_FILEPRINTPREVIEW, CM_FILEPRINTPREVIEW));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_HELPCONTENTS, CM_HELPCONTENTS));
  flatcombo = new JPFlatComboBox(GetMainWindow(), ID_COMBOBOX, 0,0,150,350, CBS_DROPDOWN, 25);
  flatcombo->SetListBoxWidth(300);
  uint32 style = flatcombo->GetStyle();
  style &= ~CBS_SORT;
  flatcombo->SetStyle(style);
  GadgetAvailableList->Insert(*new JPControlGadget(*flatcombo));
  FlatColorPicker = new JPFlatColorPicker(CLR_DEFAULT, IDS_COLORPICKER, 11, 15, TColor::Black, frame, 0, NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, ID_COLORPICKER, ID_COLORPICKER);
  GadgetAvailableList->Insert(*FlatColorPicker);
  cb->SetCaption("Main Toolbar");
  cb->SetHintMode(TGadgetWindow::EnterHints);
  cb->Attr.Id = IDW_TOOLBAR;
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_TOOLBARCUSTOMIZE, CM_TOOLBARCUSTOMIZE));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_TOOLBARCUSTOMIZE, CM_TOOLBARBITMAP));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_TOOLBARCUSTOMIZE, CM_TOOLBARTEXT));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_TOOLBARCUSTOMIZE, CM_TOOLBARBITMAPTEXT));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_TOOLBARCUSTOMIZE, CM_TOOLBARLEFT));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_TOOLBARCUSTOMIZE, CM_TOOLBARRIGHT));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_TOOLBARCUSTOMIZE, CM_TOOLBARTOP));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, CM_TOOLBARCUSTOMIZE, CM_TOOLBARBOTTOM));
  custombar->SetCaption("Customize Toolbar");
  custombar->SetHintMode(TGadgetWindow::EnterHints);

  /*      STEP 3 :  LET TOOLBARLIST KNOWS HOW MANY TOOLBARS IT MUST SUPPORT             */
  /*      Note: respect the same order as in the GadgetList array.                      */
  /*      Here below 'cb' has index 0, 'custombar' has index 1, 'mydock' has index 2.   */
  ToolbarList.Add(cb);
  ToolbarList.Add(custombar);
  ToolbarList.Add(mydock);

  //  We can retrieve the data for the color picker gadget.
  RetrieveColorPickerDataFromRegistry();

  //  Now we can set the reference gadget for all dividers.
  if (gadgid)
     {
     TGadget* gadg = GadgetAvailableList->GadgetWithId(gadgid);
     if (gadg)
        SetDivId(gadg);
     }

  /*   Create the toolbar menu and passes it to the flat button menu     */
  tbarmenu = new TMenu(*this, IDM_FASTPOPMENU);
  uint maxsize = ToolbarList.GetItemsInContainer();
  uint count = 0;
  char szBuf[255];
  for (uint i = 0; i < maxsize; i++)
      {
      TGadgetWindow* wind = ToolbarList[i];
      if (wind)
         {
         wind->GetWindowText(szBuf, sizeof(szBuf));
         bool result = (bool) tbarmenu->InsertMenu(i, MF_BYPOSITION | MF_ENABLED, CM_FIRSTTOOLBARSITEM + i, wind->Title);
         if (result)
            {
            tbarmenu->EnableMenuItem(CM_FIRSTTOOLBARSITEM + i, MF_BYCOMMAND | MF_ENABLED);
            count++;
            }
         }
      }
  if (count > 0)
     tbarmenu->InsertMenu(count, MF_BYPOSITION | MF_SEPARATOR, -1, NULL);
  GadgetAvailableList->Insert(*new JPFlatMenuButtonTextGadget(*tbarmenu, NULL, dispType, textPos, MaxChars, ToolbarRows, moreParams, ID_BUTTONMENU, ID_BUTTONMENU));
#pragma warn .eas
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Application main window construction & intialization.
//
void TBUTTONApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
     nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;
  oldCmdShow = nCmdShow;
  nCmdShow = SW_HIDE;
  MdiClient = new TBUTTONMDIClient(this);
  frame = new JPDecMDIFrame(Name, IDM_MDI, *MdiClient, true, this);
  frame->Attr.ExStyle |= WS_EX_ACCEPTFILES;
  frame->SetKeyboardHandling(true);
  frame->SetIcon(this, IDI_MDIAPPLICATION);
  frame->SetIconSm(this, IDI_MDIAPPLICATION);
  frame->Attr.AccelTable = IDM_MDI;

/*********      CREATE A "TIMES NEW ROMAN" FONT FOR ALL TOOLBARS      **********/
  LOGFONT MainFontRec;
  MainFontRec.lfHeight = -11;
  MainFontRec.lfWidth = 0;
  MainFontRec.lfEscapement = 0;
  MainFontRec.lfOrientation = 0;
  MainFontRec.lfWeight = FW_NORMAL;
  MainFontRec.lfItalic = 0;
  MainFontRec.lfUnderline = 0;
  MainFontRec.lfStrikeOut = 0;
  MainFontRec.lfCharSet = 0;
  MainFontRec.lfOutPrecision = OUT_STROKE_PRECIS;
  MainFontRec.lfClipPrecision = CLIP_STROKE_PRECIS;
  MainFontRec.lfQuality = DRAFT_QUALITY;
  MainFontRec.lfPitchAndFamily = FF_ROMAN | VARIABLE_PITCH;
  strcpy(MainFontRec.lfFaceName, "Times New Roman");
  toolbarFont = new TFont(&MainFontRec);
  LOGFONT statusbarFont = MainFontRec;
  statusbarFont.lfHeight = -13;
  TFont* sbarFont = new TFont(&statusbarFont);

/*******     CREATE THE AVAILABLE LIST GADGET WINDOW           ***********/
  GadgetAvailableList = new TGadgetWindow();
/*******     FILL THE AVAILABLE LIST AND OTHER INFO      **********/
  SetupSpeedBar(frame, TDockableControlBar::Rectangular, toolbarFont);

  TStatusBar* sb = new TStatusBar(frame, TGadget::Recessed,
                                  TStatusBar::SizeGrip        |
                                  TStatusBar::CapsLock        |
                                  TStatusBar::NumLock         |
                                  TStatusBar::ScrollLock, sbarFont);
  sb->Insert(*new TSeparatorGadget(6));
  txtGadg = new JPFlatTextGadget(CM_CURRCHARS, TGadget::Recessed, TTextGadget::Left, 5, "Chars = 4");
  sb->Insert(*txtGadg);
  sb->SetWideHints(false);
  sb->SetCaption("Status Bar");
  sb->SetHintMode(TGadgetWindow::EnterHints);
  sb->EnableTooltip(true);
  frame->Insert(*sb, TDecoratedFrame::Bottom);

  SetMainWindow(frame);
  frame->SetMenuDescr(TMenuDescr(IDM_MDI));
}

int32 TBUTTONApp::CmButtonMenuSelected(TParam1 param1, TParam2 param2)
{
  if ((int) param2 == (int) ID_BUTTONMENU)
     {
     TMenu* menu = (TMenu*) param1;
     menu->InsertMenu(0, MF_BYPOSITION, 200, "Menu Test");
     }
  return 0;
}

int32 TBUTTONApp::CmColorPickerSelected(TParam1 param1, TParam2 param2)
{
  int id = (int) param1;
  if (id == (int) ID_COLORPICKER)
     {
     TColor color = TColor((COLORREF) param2);
     uint8 flags = color.Flags();
     char buf[255];
     if (flags == 0)
        wsprintf(buf, "Flat Color Picker Notification  ID = %i.\nThe user has selected the color RGB(%u, %u, %u).", id, color.Red(), color.Green(), color.Blue());
     else
        wsprintf(buf, "Flat Color Picker Notification  ID = %i.\nThe user has selected the Default Color.", id);
     GetMainWindow()->MessageBox(buf, "Color Picker Notification", MB_OK);
     }
  return 0;
}

/********************************************************************************
*****    LOAD THE CURRENT POSITION OF THE MAIN WINDOW FROM THE REGISTRY.    *****
********************************************************************************/
void TBUTTONApp::RetrieveMainWindowInfoFromRegistry()
{
   TWindow* mw = GetMainWindow();
   if (mw && (mw->GetHandle()))
      {
      // I don't use the TWindow's GetWindowPlacement method as it doesn't initialize
      // the "length" field in WINDOWPLACEMT structure!!
      WINDOWPLACEMENT wp;
      bool useshowwindow = true;
      HKEY mwKey;
      mwKey = CreateRegistrySubkey(GetRegistryRootKey(), OptionsKey);
      int flags = GetRegistryIntegerValue(mwKey, MWFlagsKey, (int) -1);
      if (flags >= 0)
         {
         TRect rect = mw->GetWindowRect();
         wp.length = sizeof(WINDOWPLACEMENT);    // TWindow forgets to initialize this field!!
         wp.flags = flags;
         wp.showCmd = GetRegistryIntegerValue(mwKey, MWShowKey,  (int) oldCmdShow);
         wp.ptMinPosition.x = GetRegistryIntegerValue(mwKey, MWMinPosXKey,  (int) -1);
         wp.ptMinPosition.y = GetRegistryIntegerValue(mwKey, MWMinPosYKey,  (int) -1);
         wp.ptMaxPosition.x = GetRegistryIntegerValue(mwKey, MWMaxPosXKey,  (int) -1);
         wp.ptMaxPosition.y = GetRegistryIntegerValue(mwKey, MWMaxPosYKey,  (int) -1);
         wp.rcNormalPosition.left = GetRegistryIntegerValue(mwKey, MWNormPosLeftKey, rect.left);
         wp.rcNormalPosition.top = GetRegistryIntegerValue(mwKey, MWNormPosTopKey, rect.top);
         wp.rcNormalPosition.right = GetRegistryIntegerValue(mwKey, MWNormPosRightKey, rect.right);
         wp.rcNormalPosition.bottom = GetRegistryIntegerValue(mwKey, MWNormPosBottomKey, rect.bottom);
         if (mw->SetWindowPlacement(&wp) != 0)
            useshowwindow = false;
         }
      if (useshowwindow)
         {
         nCmdShow = oldCmdShow;
         nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;
         mw->ShowWindow(nCmdShow);
         }
      }
}

/********************************************************************************
*****     STORE THE CURRENT POSITION OF THE MAIN WINDOW ON THE REGISTRY.    *****
********************************************************************************/
void TBUTTONApp::StoreMainWindowInfoOnRegistry()
{
   TWindow* mw = GetMainWindow();
   if (mw && (mw->GetHandle()))
      {
      WINDOWPLACEMENT wp;
      wp.length = sizeof(WINDOWPLACEMENT);  // TWindow forgets to initialize this field!!
      if (mw->GetWindowPlacement(&wp) != 0)
         {
         HKEY mwKey;
         mwKey = CreateRegistrySubkey(GetRegistryRootKey(), OptionsKey);
         SetRegistryIntegerValue(mwKey, MWFlagsKey, (int) wp.flags);
         SetRegistryIntegerValue(mwKey, MWShowKey,  (int) wp.showCmd);
         SetRegistryIntegerValue(mwKey, MWMinPosXKey,  (int) wp.ptMinPosition.x);
         SetRegistryIntegerValue(mwKey, MWMinPosYKey,  (int) wp.ptMinPosition.y);
         SetRegistryIntegerValue(mwKey, MWMaxPosXKey,  (int) wp.ptMaxPosition.x);
         SetRegistryIntegerValue(mwKey, MWMaxPosYKey,  (int) wp.ptMaxPosition.y);
         SetRegistryIntegerValue(mwKey, MWNormPosLeftKey,  (int) wp.rcNormalPosition.left);
         SetRegistryIntegerValue(mwKey, MWNormPosTopKey,  (int) wp.rcNormalPosition.top);
         SetRegistryIntegerValue(mwKey, MWNormPosRightKey,  (int) wp.rcNormalPosition.right);
         SetRegistryIntegerValue(mwKey, MWNormPosBottomKey,  (int) wp.rcNormalPosition.bottom);
         }
      }
}

/******************************************************************************
*****    LOAD THE COLOR PICKER'S DATA FROM THE REGISTRY.                  *****
******************************************************************************/
void TBUTTONApp::RetrieveColorPickerDataFromRegistry()
{
   HKEY mwKey = CreateRegistrySubkey(GetRegistryRootKey(), ColorPickerKey);
   char bufred[25], bufgreen[25], bufblue[25], bufflag[25];
   for (int i = 0; i < 17; i++)
      {
      if (i == 0)
         {
         wsprintf(bufred, CPRedValue);
         wsprintf(bufgreen, CPGreenValue);
         wsprintf(bufblue, CPBlueValue);
         wsprintf(bufflag, CPFlagValue);
         }
      else
         {
         wsprintf(bufred, "%s%i", CPRedValue, i - 1);
         wsprintf(bufgreen, "%s%i", CPGreenValue, i - 1);
         wsprintf(bufblue, "%s%i", CPBlueValue, i - 1);
         wsprintf(bufflag, "%s%i", CPFlagValue, i - 1);
         }
      int red = GetRegistryIntegerValue(mwKey, bufred,  (int) 0);
      int green = GetRegistryIntegerValue(mwKey, bufgreen,  (int) 0);
      int blue = GetRegistryIntegerValue(mwKey, bufblue,  (int) 0);
      int flag = GetRegistryIntegerValue(mwKey, bufflag,  (int) -1);
      TColor color;
      if (i == 0)
         {
         if (flag < 0)
            color = CLR_DEFAULT;
         else
            color = TColor(red, green, blue, flag);
         FlatColorPicker->SetCurrentReferenceColor(color);
         }
      else
         {
         if (flag >= 0)
            {
            color = TColor(red, green, blue, 0);
            FlatColorPicker->SetPickerCustomColorsValue(i - 1, color);
            }
         }
      }
}

/******************************************************************************
*****    STORE THE COLOR PICKER'S DATA ON THE REGISTRY.                   *****
******************************************************************************/
void TBUTTONApp::StoreColorPickerDataOnRegistry()
{
   HKEY mwKey = CreateRegistrySubkey(GetRegistryRootKey(), ColorPickerKey);
   char bufred[25], bufgreen[25], bufblue[25], bufflag[25];
   for (int i = 0; i < 17; i++)
      {
      if (i == 0)
         {
         wsprintf(bufred, CPRedValue);
         wsprintf(bufgreen, CPGreenValue);
         wsprintf(bufblue, CPBlueValue);
         wsprintf(bufflag, CPFlagValue);
         }
      else
         {
         wsprintf(bufred, "%s%i", CPRedValue, i - 1);
         wsprintf(bufgreen, "%s%i", CPGreenValue, i - 1);
         wsprintf(bufblue, "%s%i", CPBlueValue, i - 1);
         wsprintf(bufflag, "%s%i", CPFlagValue, i - 1);
         }
      TColor color;
      if (i == 0)
         color = FlatColorPicker->GetCurrentReferenceColor();
      else
         color = FlatColorPicker->GetPickerCustomColorsValue(i - 1);
      SetRegistryIntegerValue(mwKey, bufred,  (int) color.Red());
      SetRegistryIntegerValue(mwKey, bufgreen,  (int) color.Green());
      SetRegistryIntegerValue(mwKey, bufblue,  (int) color.Blue());
      SetRegistryIntegerValue(mwKey, bufflag,  (int) color.Flags());
      }
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Application instance initialization.
//
void TBUTTONApp::InitInstance()
{
  TApplication::InitInstance();

  ProcessCmdLine(lpCmdLine);

/***    LOAD THE MAIN WINDOW'S DATA FROM THE REGISTRY AND CHANGE THEM      *****/
  RetrieveMainWindowInfoFromRegistry();

/***    LOAD THE TOOLBAR DATA FROM THE PROFILE (BEFORE) AND FROM THE ARRAY (AFTER)     *****/
  LoadToolbarDataFromProfile();

/**********   ADDED TO DISPLAY SOME GADGETS' DATA IN COMBOBOX   ************/
  char buf[255];
  int i = 0;
  if (cb)
      {
      for (TGadget* g = cb->FirstGadget(); g; g = cb->NextGadget(*g))
        {
        i++;
        int id = g->GetId();
        wsprintf(buf, "Pos=%.2i   Id=%i", i, id);
        flatcombo->AddString(buf);
        }
      if (i > 0)
         flatcombo->SetSelIndex(0);
      }

#if defined(CURRENT_WIN_VERSION)
#if defined(BI_PLAT_WIN16)
   DWORD dwVersion = ::GetVersion();
   wsprintf(buf, "Windows %u.%u\nDOS %u.%u", LOBYTE(LOWORD(dwVersion)),HIBYTE(LOWORD(dwVersion)),HIBYTE(HIWORD(dwVersion)),LOBYTE(HIWORD(dwVersion)));
#elif defined(BI_PLAT_WIN32)
   OSVERSIONINFO os;
   os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
   ::GetVersionEx(&os);
   char version[255], build[255], buildver[255], pid[255];
   wsprintf(version, "Major/Minor : %u.%u", os.dwMajorVersion, os.dwMinorVersion);
   wsprintf(build, "Build : %u", LOWORD(os.dwBuildNumber));
   wsprintf(buildver, "BuildVer : %u.%u", HIBYTE(HIWORD(os.dwBuildNumber)), LOBYTE(HIWORD(os.dwBuildNumber)));
   wsprintf(pid, "Platform ID : %u", os.dwPlatformId);
   wsprintf(buf, "%s\n%s\n%s\n%s\nCSDVersion : <%s>", version, build, buildver, pid, os.szCSDVersion);
#endif
  GetMainWindow()->MessageBox(buf, "Current Windows Info", MB_OK);
#endif
}

//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Process command line parameters.
//
void TBUTTONApp::ProcessCmdLine(char * CmdLine)
{
  TCmdLine cmd(CmdLine);
  char buf[_MAX_PATH];

  while (cmd.Kind != TCmdLine::Done) {
    if (cmd.Kind == TCmdLine::Option) {
      if (strnicmp(cmd.Token, "unregister", cmd.TokenLen) == 0) {
        UnRegisterInfo();
        return;
      }
    }
/*******    ADDED TO SUPPORT FILENAMES PASSED IN COMMAND LINE    **********/
    if (cmd.Kind == TCmdLine::Name)
        {
        strncpy(buf, cmd.Token, cmd.TokenLen);
        buf[cmd.TokenLen] = '\0';
        TDocTemplate* tpl = GetDocManager()->MatchTemplate(buf);
        if (tpl)
          GetDocManager()->CreateDoc(tpl, buf);
        }
    cmd.NextToken(true);
  }

  RegisterInfo();
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Register application info.
//
void TBUTTONApp::RegisterInfo()
{
  TAPointer<char> buffer = new char[_MAX_PATH];

  GetModuleFileName(buffer, _MAX_PATH);

  TRegKey(TRegKey::ClassesRoot, "BUTTON.Application\\DefaultIcon").SetDefValue(0, REG_SZ, buffer, strlen(buffer));
  strcat(buffer, ",1");
  TRegKey(TRegKey::ClassesRoot, "BUTTON.Document.1\\DefaultIcon").SetDefValue(0, REG_SZ, buffer, strlen(buffer));
  strcpy(buffer, "BUTTON.Document.1");
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Unregister application info.
//
void TBUTTONApp::UnRegisterInfo()
{
  TAPointer<char> buffer = new char[_MAX_PATH];

  GetModuleFileName(buffer, _MAX_PATH);

  TRegKey(TRegKey::ClassesRoot, "BUTTON.Application").DeleteKey("DefaultIcon");
  TRegKey(TRegKey::ClassesRoot, "BUTTON.Document.1").DeleteKey("DefaultIcon");

  TRegKey::ClassesRoot.DeleteKey("BUTTON.Application");
  TRegKey::ClassesRoot.DeleteKey("BUTTON.Document.1");
}



//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Response Table handlers:
//
void TBUTTONApp::EvNewView(TView& view)
{
  TMDIClient* mdiClient = TYPESAFE_DOWNCAST(GetMainWindow()->GetClientWindow(), TMDIClient);
  if (mdiClient) {
    TBUTTONMDIChild* child = new TBUTTONMDIChild(*mdiClient, 0, view.GetWindow());

    if (view.GetViewMenu())
      child->SetMenuDescr(*view.GetViewMenu());

    child->SetIcon(this, IDI_DOC);
    child->SetIconSm(this, IDI_DOC);
    child->Create();
  }
}


void TBUTTONApp::EvCloseView(TView&)
{
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Menu Help Contents command
//
void TBUTTONApp::CmHelpContents()
{
  HelpState = GetMainWindow()->WinHelp(HelpFileName, HELP_CONTENTS, 0);
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Menu Help Using Help command
//
void TBUTTONApp::CmHelpUsing()
{
  HelpState = GetMainWindow()->WinHelp(HelpFileName, HELP_HELPONHELP, 0);
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~~~~~~~
// Menu Help About BUTTON command
void TBUTTONApp::CmHelpAbout()
{
  TBUTTONAboutDlg(GetMainWindow()).Execute();
}


void TBUTTONApp::EvDropFiles(TDropInfo drop)
{
  TFileDropletList files;

  int fileCount = drop.DragQueryFileCount();
  for (int i = 0; i < fileCount; i++)
    files.Add(new TFileDroplet(drop, i));

  AddFiles(files);

  drop.DragFinish();
}


void TBUTTONApp::AddFiles(TFileDropletList& files)
{
  for (TFileDropletListIter fileIter(files); fileIter; fileIter++) {
    TDocTemplate* tpl = GetDocManager()->MatchTemplate(fileIter.Current()->GetName());
    if (tpl)
      GetDocManager()->CreateDoc(tpl, fileIter.Current()->GetName());
  }
}


void TBUTTONApp::EvOwlDocument(TDocument& doc)
{
  if (doc.GetDocPath())
    SaveMenuChoice(doc.GetDocPath());
}

int32 TBUTTONApp::CmFileSelected(uint wp, int32)
{
  TAPointer<char> text = new char[_MAX_PATH];

  GetMenuText(wp, text, _MAX_PATH);
  TDocTemplate* tpl = GetDocManager()->MatchTemplate(text);
  if (tpl)
    GetDocManager()->CreateDoc(tpl, text);
  return 0;
}

bool TBUTTONApp::ProcessAppMsg(MSG& msg)
{
  if (msg.message == WM_COMMAND) {
    if (ContextHelp || ::GetKeyState(VK_F1) < 0) {
      ContextHelp = false;
      GetMainWindow()->WinHelp(HelpFileName, HELP_CONTEXT, msg.wParam);
      return true;
    }
  }
  else
    switch(msg.message) {
    case WM_KEYDOWN:
      if (msg.wParam == VK_F1) {
        if (::GetKeyState(VK_SHIFT) < 0) {
          ContextHelp = true;
          HelpCursor = new TCursor(GetMainWindow()->GetModule()->GetInstance(), TResId(IDC_HELPCURSOR));
          ::SetCursor(*HelpCursor);
          return true;
        }
        else {
          GetMainWindow()->WinHelp(HelpFileName, HELP_INDEX, 0);
          return true;
        }
      }
      else {
        if (ContextHelp && msg.wParam == VK_ESCAPE) {
          if (HelpCursor)
            delete HelpCursor;
          HelpCursor = 0;
          ContextHelp = false;
          GetMainWindow()->SetCursor(0, IDC_ARROW);
          return true;
        }
      }
      break;

    case WM_MOUSEMOVE:
    case WM_NCMOUSEMOVE:
      if (ContextHelp) {
        ::SetCursor(*HelpCursor);
        return true;
      }
      break;

    case WM_INITMENU:
      if (ContextHelp) {
        ::SetCursor(*HelpCursor);
        return true;
      }
      break;
    case WM_ENTERIDLE:
      if (msg.wParam == MSGF_MENU)
        if (GetKeyState(VK_F1) < 0) {
          ContextHelp = true;
          GetMainWindow()->PostMessage(WM_KEYDOWN, VK_RETURN, 0);
          return true;
        }
      break;
    default:
      ;
    }

  return TApplication::ProcessAppMsg(msg);
}


void TBUTTONApp::EvWinIniChange(char far* section)
{
  if (strcmp(section, "windows") == 0) {
    if (Printer) {
      const int bufferSize = 255;
      char printDBuffer[bufferSize];
      LPSTR printDevice = printDBuffer;
      LPSTR devName;
      LPSTR driverName = 0;
      LPSTR outputName = 0;
      if (::GetProfileString("windows", "device", "", printDevice, bufferSize)) {
        devName = printDevice;
        while (*printDevice) {
          if (*printDevice == ',') {
            *printDevice++ = 0;
            if (!driverName)
              driverName = printDevice;
            else
              outputName = printDevice;
          }
          else
            printDevice = ::AnsiNext(printDevice);
        }

        if (Printer->GetSetup().Error != 0                ||
            strcmp(devName, Printer->GetSetup().GetDeviceName()) != 0  ||
            strcmp(driverName, Printer->GetSetup().GetDriverName()) != 0 ||
            strcmp(outputName, Printer->GetSetup().GetOutputName()) != 0 ) {
          delete Printer;
          Printer = new TPrinter(this);
        }
      }
      else {
        delete Printer;
        Printer = new TPrinter(this);
      }
    }
  }
}


int OwlMain(int , char* [])
{
  TBUTTONApp   app;
  return app.Run();
}


/********************************************************************************
***                                                                           ***
***           NEW CODE FOR TOOLBARS SUPPORT                                   ***
***                                                                           ***
***   (DON'T DELETE THESE LINES : THEY ARE CALLED BY CUSTOMIZE DIALOG).       ***
***                                                                           ***
********************************************************************************/

bool TBUTTONApp::InsertGadgetInToolbar(int gadgid, TGadgetWindow* wind, TGadgetWindow::TPlacement place, TGadget* gadgpos)
{
  bool result = false;

  if (wind)
     {
     if (gadgid > 0)
        {
        TGadget* gadg = GadgetAvailableList->GadgetWithId(gadgid);
        if (gadg)
            {
            TGadget* gadgmove = GadgetAvailableList->Remove(*gadg);
            if (gadgmove)
               {
               TControlGadget* ctlgadg = TYPESAFE_DOWNCAST(gadgmove, TControlGadget);
               if (ctlgadg)    //  If the gadget is a control, show this control
                  {
                  TWindow* window = ctlgadg->GetControl();
                  if ((window) && (wind->IsWindowVisible()))
                     window->ShowWindow(SW_SHOW);
                  }
               if (gadgpos == 0)
                  wind->Insert(*gadgmove);
               else wind->Insert(*gadgmove, place, gadgpos);
               result = true;
               }
            }
        }
     else
        {
        result = true;
        if (gadgid == 0)              // insert a divider gadget
           {
           if (gadgpos)
              wind->Insert(*new JPFlatDivGadget(0, divGadget, divEOL, divVertBar), place, gadgpos);
           else
              wind->Insert(*new JPFlatDivGadget(0, divGadget, divEOL, divVertBar));
           }
        else if (gadgid == (int) -1)  // insert a docking grip
                {
                if (gadgpos)
                   wind->Insert(*new JPFlatGripGadget(false), place, gadgpos);
                else
                   wind->Insert(*new JPFlatGripGadget(false));
                }
        }
     }

  return result;
}

bool TBUTTONApp::RemoveGadgetFromToolbar(TGadget* gadg, TGadgetWindow* wind, TGadgetWindow::TPlacement place, TGadget* gadgpos)
{
  bool result = false;

  if ((gadg) && (wind))
     {
     int id = gadg->GetId();
     TGadget* gadgmove = wind->Remove(*gadg);
     if (gadgmove)
         {
         TControlGadget* ctlgadg = TYPESAFE_DOWNCAST(gadgmove, TControlGadget);
         if (ctlgadg)    //  If the gadget is a control, hides this control
            {
            TWindow* window = ctlgadg->GetControl();
            if (window)
               window->ShowWindow(SW_HIDE);
            }
         if (id > 0)
            {
            if (gadgpos == 0)
               GadgetAvailableList->Insert(*gadgmove);
            else GadgetAvailableList->Insert(*gadgmove, place, gadgpos);
            }
         else
            delete gadgmove;
         result = true;
         }
     }

  return result;
}

void TBUTTONApp::InsertDockingGripInToolbar(TGadgetWindow* wind)
{
    if (wind)               // Force the grip to be the first gadget in the toolbar.
       {
       TGadget* firstgadg = wind->FirstGadget();
       InsertGadgetInToolbar((int) -1, wind, TGadgetWindow::Before, firstgadg);
       }
}

void TBUTTONApp::RemoveDockingGripFromToolbar(TGadgetWindow* wind)
{
  if (wind)               // Force the grip to be the first gadget in the toolbar.
     {
     bool canCont = true;
     while (canCont)
          {
          TGadget* gadg = wind->GadgetWithId((int) -1);
          if (gadg)
             canCont = RemoveGadgetFromToolbar(gadg, wind);
          else canCont = false;
          }
     }
}

void TBUTTONApp::InsertDockingGripInAllToolbars(bool updatewindow)
{
   for (uint i = 0; i < ToolbarList.GetItemsInContainer(); i++)
      {
      TGadgetWindow* wind = ToolbarList[i];
      if (wind)
         {
         InsertDockingGripInToolbar(wind);
         if (updatewindow)
            {
            wind->Invalidate();
            wind->LayoutSession();
            }
         }
      }
}

void TBUTTONApp::RemoveDockingGripFromAllToolbars(bool updatewindow)
{
   for (uint i = 0; i < ToolbarList.GetItemsInContainer(); i++)
      {
      TGadgetWindow* wind = ToolbarList[i];
      if (wind)
         {
         RemoveDockingGripFromToolbar(wind);
         if (updatewindow)
            {
            wind->Invalidate();
            wind->LayoutSession();
            }
         }
      }
}

/***   Insert the default separator at the top of the available list    ***/
void TBUTTONApp::InsertSeparatorInAvailableList()
{
  TGadget* firstgadg = GadgetAvailableList->FirstGadget();
  InsertGadgetInToolbar(0, GadgetAvailableList, TGadgetWindow::Before, firstgadg);
}

void TBUTTONApp::RemoveSeparatorFromAvailableList()
{
  bool canCont = true;
  while (canCont)
       {
       TGadget* gadg = GadgetAvailableList->GadgetWithId(0);
       if (gadg)
          canCont = RemoveGadgetFromToolbar(gadg, GadgetAvailableList);
       else canCont = false;
       }
}

void TBUTTONApp::StoreToolbarDataInProfile()  // Now it supports the registry.
{
   uint count;
   char sectbuf[255];
   char keybuf[255];
   HKEY tbarKey;
   // Store the toolbars' options on the registry.
   tbarKey = CreateRegistrySubkey(GetRegistryRootKey(), ToolbarOptions);
   SetRegistryIntegerValue(tbarKey, TOOptionalParams, (int) moreParams);
   SetRegistryIntegerValue(tbarKey, TODisplayType, (int) dispType);
   SetRegistryIntegerValue(tbarKey, TOTextPos, (int) textPos);
   SetRegistryIntegerValue(tbarKey, TOToolbarRows, (int) ToolbarRows);
   SetRegistryIntegerValue(tbarKey, TOMaxChars, (int) MaxChars);
   SetRegistryIntegerValue(tbarKey, TOHotTrack, (int) HotTrack);
   SetRegistryIntegerValue(tbarKey, TODivEol, (int) divEOL);
   SetRegistryIntegerValue(tbarKey, TODivVertBar, (int) divVertBar);
   SetRegistryIntegerValue(tbarKey, TODockingGrip, (int) DockingGrip);
   int gadgid;
   if (divGadget)
      gadgid = divGadget->GetId();
   else gadgid = 0;
   SetRegistryIntegerValue(tbarKey, TODivRefGadget, (int) gadgid);
   // store the data for each toolbar.
   for (uint i = 0; i < ToolbarList.GetItemsInContainer(); i++)
      {
      wsprintf(sectbuf, "%s%u", ToolbarKey, i);
      tbarKey = CreateRegistrySubkey(GetRegistryRootKey(), sectbuf);
      TGadgetWindow* wind = ToolbarList[i];
      if (wind)
         {
         TWindow* parentwindow = wind->GetParentO();
         if (parentwindow)
            {
            TAbsLocation docklocation;
            TRect rect (0,0,0,0);
            TDockingSlip* slip = TYPESAFE_DOWNCAST(parentwindow,TDockingSlip);
            if (slip)
              {         // The toolbar is in the harbor (it's visible)
              docklocation = slip->GetLocation();
              if (docklocation == alNone)
                 {      // The toolbar is floating.
                 TFloatingSlip* flslip = TYPESAFE_DOWNCAST(slip, TFloatingSlip);
                 if (flslip)
                    flslip->GetRect(rect);
                 }
              else  // The toolbar is on one of the four edges.
                 {
                 THarbor* harbor = slip->GetHarbor();
                 if (harbor)
                    wind->GetWindowRect(rect);
                 }
              }
            else    // The Toolbar is not in the harbor (it's hidden).
                 {
                 wind->GetWindowRect(rect);
                 docklocation = alNone;
                 }
            // Finally we can store the data on the registry.
            SetRegistryIntegerValue(tbarKey, LocationKey, (int) docklocation);
            SetRegistryIntegerValue(tbarKey, PosXKey, (int) rect.left);
            SetRegistryIntegerValue(tbarKey, PosYKey, (int) rect.top);
            SetRegistryIntegerValue(tbarKey, WidthKey, (int) rect.Width());
            SetRegistryIntegerValue(tbarKey, HeightKey, (int) rect.Height());
            SetRegistryIntegerValue(tbarKey, VisibleKey, (int) wind->IsWindowVisible());
            }
         count = 0;
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           int id = g->GetId();
           if (id >= 0)
              {
              wsprintf(keybuf, "%s%u", ButtonKey, count);
              SetRegistryIntegerValue(tbarKey, keybuf, id);
              count++;
              }
           }
         // delete the remaining keys
         bool endofsection = false;
         while (endofsection == false)
           {
           wsprintf(keybuf, "%s%u", ButtonKey, count);
           int gadgid = GetRegistryIntegerValue(tbarKey, keybuf, -1);
           if (gadgid != -1)
              DeleteRegKey(tbarKey, keybuf);
           else endofsection = true;
           count++;
           }
         }
      }
}

// This method retrieves toolbars' data info from the registry and disposes
// the toolbars automatically, in the same order and position as they were stored.
void TBUTTONApp::LoadToolbarDataFromProfile()
{
#pragma warn -bei
   RemoveSeparatorFromAvailableList();
   uint maxsize = ToolbarList.GetItemsInContainer();
   if (maxsize > 0)
      {
      uint count;
      char sectbuf[255];
      char keybuf[255];
      TBarStructEx* tbarstruct = new TBarStructEx[maxsize];
      int topcount = 0;
      for (uint i = 0; i < maxsize; i++)
         {
         wsprintf(sectbuf, "%s%u", ToolbarKey, i);
         HKEY tbarKey = CreateRegistrySubkey(GetRegistryRootKey(), sectbuf);
         TGadgetWindow* wind = ToolbarList[i];
         bool onekeyfound = false;
         if (wind)
            {
            if (DockingGrip)
               InsertDockingGripInToolbar(wind);
            count = 0;
            bool endofsection = false;
            while (endofsection == false)
              {
              wsprintf(keybuf, "%s%u", ButtonKey, count);
              int gadgid = GetRegistryIntegerValue(tbarKey, keybuf, -1);
              if (gadgid != -1)
                 {
                 if ((InsertGadgetInToolbar(gadgid, wind)) && (onekeyfound == false))
                    onekeyfound = true;
                 }
              else endofsection = true;
              count++;
              }
            if (!onekeyfound)
               RetrieveSingleToolbarFromArray(i, wind);
            bool visible = (bool) GetRegistryIntegerValue(tbarKey, VisibleKey, 1);
            if (visible)  // the toolbar is visible.
               {
               TPoint pt = TPoint(0,0);
               TSize  size = TSize(0,0);
               pt.x = GetRegistryIntegerValue(tbarKey, PosXKey, 0);
               pt.y = GetRegistryIntegerValue(tbarKey, PosYKey, 0);
               size.cx = GetRegistryIntegerValue(tbarKey, WidthKey, 0);
               size.cy = GetRegistryIntegerValue(tbarKey, HeightKey, 0);
               int windowloc = GetRegistryIntegerValue(tbarKey, LocationKey, (int) -1);
               if (windowloc < 0)
                  {
                  TWindow* parwind = ApxHarbor->GetParentO();
                  if (parwind)
                     parwind->ClientToScreen(pt);
                  windowloc = GetDefaultLocation(i);
                  }
               tbarstruct[topcount].TBarData = (int) i;
               tbarstruct[topcount].pt = pt;
               tbarstruct[topcount].loc = (TAbsLocation) windowloc;
               tbarstruct[topcount].size = size;
               topcount++;
               }
            else wind->ShowWindow(SW_HIDE); // hides the toolbar.
            }
         }
      if (topcount > 0)  // Dispose all toolbars automatically
         {
         DisposeToolbarsByArray(tbarstruct, topcount, alNone);
         DisposeToolbarsByArray(tbarstruct, topcount, alTop);
         DisposeToolbarsByArray(tbarstruct, topcount, alLeft);
         DisposeToolbarsByArray(tbarstruct, topcount, alRight);
         DisposeToolbarsByArray(tbarstruct, topcount, alBottom);
         }
      delete[] tbarstruct;
      }
   InsertSeparatorInAvailableList();
#pragma warn .bei
}

// Dispose only toolbars that are in the specified location. We must do this as edge-docked
// toolbars don't layout if you change the size of each toolbar
// (it is not needed if toolbar is floating).
void TBUTTONApp::DisposeToolbarsByArray(TBarStructEx* tbarex, int tbarcount, TAbsLocation location)
{
   TBarStruct* tbarstruct = new TBarStruct[tbarcount];
   int curcount = 0;
   // Retrieve the array for the single specified location.
   for (int i = 0; i < tbarcount; i++)
      {
      if (tbarex[i].loc == location)
         {
         tbarstruct[curcount].TBarData = tbarex[i].TBarData;
         tbarstruct[curcount].pt = tbarex[i].pt;
         tbarstruct[curcount].size = tbarex[i].size;
         curcount++;
         }
      }
   if (curcount > 0)
      {
      // If in this location there are toolbars, sort them according to the location.
      // We don't care if location is alNone.
      if (location != alNone)
         {
         TBarStruct tempstruct;
         for (int i = 0; i < curcount; i++)
            {
            for (int j = i + 1; j < curcount; j++)
               {
               if (((location == alTop) && (tbarstruct[j].pt.y < tbarstruct[i].pt.y)) ||
                  ((location == alBottom) && (tbarstruct[i].pt.y < tbarstruct[j].pt.y)) ||
                  ((location == alLeft) && (tbarstruct[j].pt.x < tbarstruct[i].pt.x)) ||
                  ((location == alRight) && (tbarstruct[i].pt.x < tbarstruct[j].pt.x)))
                  {
                  tempstruct = tbarstruct[j];
                  tbarstruct[j] = tbarstruct[i];
                  tbarstruct[i] = tempstruct;
                  }
               }
            }
         }
      int sourceref = -1;
      int destref = -1;
      TDockable* prevwin = NULL;
      // Dispose the toolbar on this location.
      for (int i = 0; i < curcount; i++)
         {
         TGadgetWindow* wind = ToolbarList[tbarstruct[i].TBarData];
         if (wind)
            {
            TDockableControlBar* cbar = TYPESAFE_DOWNCAST(wind, TDockableControlBar);
            if (cbar)
               {
               cbar->Attr.Style |= WS_VISIBLE;
               cbar->ShowWindow((int) cbar->Attr.Style);
               TPoint pt = tbarstruct[i].pt;
               TPoint point = pt;
               if (((pt.y != sourceref) && (location == alTop)) ||
                   ((pt.x != sourceref) && (location == alLeft)) ||
                   ((pt.x != sourceref) && (location == alRight)))
                  {
                  if (sourceref >= 0)
                     {
                     TRect rect = prevwin->GetWindow()->GetWindowRect();
                     TRelPosition relpos = rpNone;
                     if (location == alTop)
                        {
                        destref = rect.bottom;
                        point.y = destref;
                        }
                     else if (location == alLeft)
                             {
                             destref = rect.right + 1;
                             point.x = destref;
                             }
                     else if (location == alRight)
                             {
                             destref = rect.left;
                             point.x = destref;
                             relpos = rpNone;
                             }
                     ApxHarbor->Insert(*cbar, location, &point, relpos, prevwin);
                     }
                  else
                     {    // the first toolbar of this location.
                     ApxHarbor->Insert(*cbar, location, &pt);
                     TRect rect = cbar->GetWindowRect();
                     if (location == alTop)
                        destref = rect.top;
                     else if (location == alLeft)
                             destref = rect.left;
                     else if (location == alRight)
                             destref = rect.left;
                     }
                  if (location == alTop)
                     sourceref = pt.y;
                  else if (location == alLeft)
                          sourceref = pt.x;
                  else if (location == alRight)
                          sourceref = pt.x;
                  prevwin = cbar;
                  }
               else
                  {
                  TRelPosition relpos = rpNone;
                  if (location == alTop)
                     pt.y = destref;
                  else if (location == alLeft)
                          {
                          pt.x = destref;
                          relpos = rpRightOf;
                          }
                  else if (location == alRight)
                          {
                          pt.x = destref - tbarstruct[i].size.cx;
                          relpos = rpNone;
                          }
                  TDockingSlip* dslip = ApxHarbor->Insert(*cbar, location, &pt, relpos, prevwin);
                  TFloatingSlip* flslip = TYPESAFE_DOWNCAST(dslip, TFloatingSlip);
                  TSize size = tbarstruct[i].size;
                  // if the toolbar is floating, we adjust its rectangle's size.
                  if ((flslip) && (location == alNone))
                     {
                     TSize newsize = flslip->ComputeSize(alNone, &size);
                     flslip->Layout(alNone, &newsize);
                     }
                  }
               }
            }
         }
      }
   delete[] tbarstruct;
}

// Dispose all toolbars. We must do this as edge-docked toolbars don't layout if you
// change the size of each toolbar (it is not needed if toolbar is floating).
void TBUTTONApp::DisposeAllToolbars()
{
   uint maxsize = ToolbarList.GetItemsInContainer();
   if (maxsize > 0)
      {
      TBarStructEx* tbarstruct = new TBarStructEx[maxsize];
      int curcount = 0;
      for (uint i = 0; i < maxsize; i++)
         {
         TGadgetWindow* wind = ToolbarList[i];
         if (wind)
            {
            TWindow* parentwindow = wind->GetParentO();
            if (parentwindow)
               {
               TAbsLocation docklocation;
               TRect rect (0,0,0,0);
               TDockingSlip* slip = TYPESAFE_DOWNCAST(parentwindow,TDockingSlip);
               if (slip)
                 {         // The toolbar is in the harbor (it's visible)
                 docklocation = slip->GetLocation();
                 if (docklocation == alNone)
                    {      // The toolbar is floating.
                    TFloatingSlip* flslip = TYPESAFE_DOWNCAST(slip, TFloatingSlip);
                    if (flslip)
                       flslip->GetRect(rect);
                    }
                 else  // The toolbar is in one of the four edges.
                    {
                    THarbor* harbor = slip->GetHarbor();
                    if (harbor)
                       wind->GetWindowRect(rect);
                    }
                 // Add to array only if toolbar is visible.
                 tbarstruct[curcount].TBarData = (int) i;
                 tbarstruct[curcount].pt = TPoint(rect.left, rect.top);
                 tbarstruct[curcount].size = TSize(rect.Width(), rect.Height());
                 tbarstruct[curcount].loc = docklocation;
                 curcount++;
                 }
               }
            }
         }
      if (curcount > 0)
         {
         DisposeToolbarsByArray(tbarstruct, curcount, alTop);
         DisposeToolbarsByArray(tbarstruct, curcount, alLeft);
         DisposeToolbarsByArray(tbarstruct, curcount, alRight);
         DisposeToolbarsByArray(tbarstruct, curcount, alBottom);
         }
      delete[] tbarstruct;
      }
}

TAbsLocation TBUTTONApp::GetDefaultLocation(uint index)
{
   return TBarsLocation[index];
}

void TBUTTONApp::RetrieveSingleToolbarFromArray(int pos, TGadgetWindow* wind)
{
  if (wind)
     {
     int maxnum = sizeof(GadgetList) / sizeof(TOOLBARDATA);
     for (int i = 0; i < maxnum; i++)
         {
         TOOLBARDATA data = GadgetList[i];
         int gadgid = data.GadgetId;
         int toolid = data.ToolbarPos;
         if ((toolid == pos) && (gadgid >= 0))
            InsertGadgetInToolbar(gadgid, wind);
         }
     }
}

void TBUTTONApp::LoadToolbarDataFromArray(bool updatewindow)
{
  int maxnum = sizeof(GadgetList) / sizeof(TOOLBARDATA);
  for (int i = 0; i < maxnum; i++)
      {
      TOOLBARDATA data = GadgetList[i];
      int gadgid = data.GadgetId;
      int toolid = data.ToolbarPos;
      TGadgetWindow* gadgwin = ToolbarList[toolid];
      if (gadgwin)
         InsertGadgetInToolbar(gadgid, gadgwin);
      }
  if (updatewindow)
      {
      for (uint i = 0; i < ToolbarList.GetItemsInContainer(); i++)
         {
         TGadgetWindow* wind = ToolbarList[i];
         if (wind)
            {
            if (DockingGrip)
               InsertDockingGripInToolbar(wind);
            wind->Invalidate();
            wind->LayoutSession();
            }
         }
      }
  InsertSeparatorInAvailableList();
}

void TBUTTONApp::ReloadToolbarData()
{
   for (uint i = 0; i < ToolbarList.GetItemsInContainer(); i++)
      {
      TGadgetWindow* wind = ToolbarList[i];
      bool canCont = true;
      if (wind)
         {
         TGadget* g;
         g = wind->FirstGadget();
         while ((g) && (canCont))
               {
               canCont = RemoveGadgetFromToolbar(g, wind);
               g = wind->FirstGadget();
               }
         }
      }
   RemoveSeparatorFromAvailableList();
   LoadToolbarDataFromArray(true);
}

TGadget* TBUTTONApp::FindGadgetInToolbars(int id)
{
   TGadget* result = NULL;
   uint maxnum = ToolbarList.GetItemsInContainer();
   TGadgetWindow* wind;
   for (uint i = 0; i <= maxnum; i++)
      {
      if (i < maxnum)
         wind = ToolbarList[i];
      else wind = GadgetAvailableList;
      if ((wind) && (!result))
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           if (g->GetId() == id)
              {
              result = g;
              break;
              }
           }
         }
      }
   return result;
}

/********************************************************************************
***                                                                           ***
***           NEW CODE FOR TOOLBARS SUPPORT                                   ***
***                                                                           ***
***   (YOU CAN DELETE THESE LINES IF YOU DON'T NEED A FULL SUPPORT).          ***
***                                                                           ***
********************************************************************************/
void TBUTTONApp::SetDisplayType(JPFlatButtonTextGadget::JPDisplayType disp)
{
   dispType = disp;
   uint maxnum = ToolbarList.GetItemsInContainer();
   TGadgetWindow* wind;
   for (uint i = 0; i <= maxnum; i++)
      {
      if (i < maxnum)
         wind = ToolbarList[i];
      else wind = GadgetAvailableList;
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatButtonTextGadget* bg = TYPESAFE_DOWNCAST(g, JPFlatButtonTextGadget);
           if (bg)
              bg->SetDisplayType(disp);
           }
         }
      }
   DisposeAllToolbars();
}

void TBUTTONApp::SetTextPosition(JPFlatButtonTextGadget::JPTextPos pos)
{
   textPos = pos;
   uint maxnum = ToolbarList.GetItemsInContainer();
   TGadgetWindow* wind;
   for (uint i = 0; i <= maxnum; i++)
      {
      if (i < maxnum)
         wind = ToolbarList[i];
      else wind = GadgetAvailableList;
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatButtonTextGadget* bg = TYPESAFE_DOWNCAST(g, JPFlatButtonTextGadget);
           if (bg)
              bg->SetTextPosition(pos);
           }
         }
      }
   DisposeAllToolbars();
}

void TBUTTONApp::SetTextRows(uint rows)
{
   ToolbarRows = rows;
   uint maxnum = ToolbarList.GetItemsInContainer();
   TGadgetWindow* wind;
   for (uint i = 0; i <= maxnum; i++)
      {
      if (i < maxnum)
         wind = ToolbarList[i];
      else wind = GadgetAvailableList;
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatButtonTextGadget* bg = TYPESAFE_DOWNCAST(g, JPFlatButtonTextGadget);
           if (bg)
              bg->SetTextRows(rows);
           }
         }
      }
   DisposeAllToolbars();
}

void TBUTTONApp::SetMaxChars(uint maxchars)
{
   MaxChars = maxchars;
   if (txtGadg)
      {
      char buf[255];
      wsprintf(buf, "Chars = %u", MaxChars);
      txtGadg->SetText(buf);
      }
   uint maxnum = ToolbarList.GetItemsInContainer();
   TGadgetWindow* wind;
   for (uint i = 0; i <= maxnum; i++)
      {
      if (i < maxnum)
         wind = ToolbarList[i];
      else wind = GadgetAvailableList;
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatButtonTextGadget* bg = TYPESAFE_DOWNCAST(g, JPFlatButtonTextGadget);
           if (bg)
              bg->SetMaxChars(maxchars);
           }
         }
      }
   DisposeAllToolbars();
}

void TBUTTONApp::SetDivId(TGadget* id)
{
   divGadget = id;
   uint maxnum = ToolbarList.GetItemsInContainer();
   TGadgetWindow* wind;
   for (uint i = 0; i <= maxnum; i++)
      {
      if (i < maxnum)
         wind = ToolbarList[i];
      else wind = GadgetAvailableList;
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatDivGadget* divg = TYPESAFE_DOWNCAST(g, JPFlatDivGadget);
           if (divg)
              divg->SetReferenceGadget(divGadget);
           }
         }
      }
   DisposeAllToolbars();
}

void TBUTTONApp::CeToolbarbitmap(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((dispType == JPFlatButtonTextGadget::Bitmap) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CeToolbarbitmaptext(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((dispType == JPFlatButtonTextGadget::BitmapText) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CeToolbarbottom(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck((textPos == JPFlatButtonTextGadget::Bottom) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CeToolbarleft(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck((textPos == JPFlatButtonTextGadget::Left) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CeToolbarright(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck((textPos == JPFlatButtonTextGadget::Right) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CeToolbartext(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((dispType == JPFlatButtonTextGadget::Text) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CeToolbartop(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck((textPos == JPFlatButtonTextGadget::Top) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TBUTTONApp::CeOneTextRow(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck((ToolbarRows == 1) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TBUTTONApp::CeTwoTextRow(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck((ToolbarRows == 2) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TBUTTONApp::CmToolbarbitmap()
{
  // INSERT>> Your code here.
  SetDisplayType(JPFlatButtonTextGadget::Bitmap);
}


void TBUTTONApp::CmToolbarbitmaptext()
{
  // INSERT>> Your code here.
  SetDisplayType(JPFlatButtonTextGadget::BitmapText);
}


void TBUTTONApp::CmToolbarbottom()
{
  // INSERT>> Your code here.
  SetTextPosition(JPFlatButtonTextGadget::Bottom);
}


void TBUTTONApp::CmToolbarleft()
{
  // INSERT>> Your code here.
  SetTextPosition(JPFlatButtonTextGadget::Left);
}


void TBUTTONApp::CmToolbarright()
{
  // INSERT>> Your code here.
  SetTextPosition(JPFlatButtonTextGadget::Right);
}


void TBUTTONApp::CmToolbartext()
{
  // INSERT>> Your code here.
  SetDisplayType(JPFlatButtonTextGadget::Text);
}


void TBUTTONApp::CmToolbartop()
{
  // INSERT>> Your code here.
  SetTextPosition(JPFlatButtonTextGadget::Top);
}

void TBUTTONApp::CmOneTextRow()
{
  // INSERT>> Your code here.
  SetTextRows(1);
}

void TBUTTONApp::CmTwoTextRow()
{
  // INSERT>> Your code here.
  SetTextRows(2);
}


void TBUTTONApp::CmDecreasechars()
{
  // INSERT>> Your code here.
  if (MaxChars > 0)
     SetMaxChars(--MaxChars);
}


void TBUTTONApp::CeDecreasechars(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if ((dispType != JPFlatButtonTextGadget::Bitmap) && (MaxChars > 0))
     tce.Enable(true);
  else tce.Enable(false);
}


void TBUTTONApp::CmDefaultchars()
{
  // INSERT>> Your code here.
  SetMaxChars(4);
}


void TBUTTONApp::CeDefaultchars(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck((MaxChars == 4) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CmIncreasechars()
{
  // INSERT>> Your code here.
  if (MaxChars <= 10)
     SetMaxChars(++MaxChars);
}


void TBUTTONApp::CeIncreasechars(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if ((dispType != JPFlatButtonTextGadget::Bitmap) && (MaxChars <= 10))
     tce.Enable(true);
  else tce.Enable(false);
}


void TBUTTONApp::CmHottracking()
{
  // INSERT>> Your code here.
   HotTrack ^= 1;
   uint maxnum = ToolbarList.GetItemsInContainer();
   TGadgetWindow* wind;
   for (uint i = 0; i <= maxnum; i++)
      {
      if (i < maxnum)
         wind = ToolbarList[i];
      else wind = GadgetAvailableList;
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatButtonTextGadget* bg = TYPESAFE_DOWNCAST(g, JPFlatButtonTextGadget);
           if (bg)
              bg->SetHotTrack(HotTrack);
           }
         }
      }
}

void TBUTTONApp::CeHottracking(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck(HotTrack ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TBUTTONApp::CmDivbuttons()
{
  // INSERT>> Your code here.
  SetDivId(FindGadgetInToolbars(CM_MDIFILENEW));
}

void TBUTTONApp::CeDivbuttons(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((divGadget == FindGadgetInToolbars(CM_MDIFILENEW)) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TBUTTONApp::CmDivcombo()
{
  // INSERT>> Your code here.
  SetDivId(FindGadgetInToolbars(ID_COMBOBOX));
}

void TBUTTONApp::CeDivcombo(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((divGadget == FindGadgetInToolbars(ID_COMBOBOX)) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TBUTTONApp::CmDivnothing()
{
  // INSERT>> Your code here.
  SetDivId(0);
}

void TBUTTONApp::CeDivnothing(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((divGadget == 0) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TBUTTONApp::CmDivendofline()
{
  // INSERT>> Your code here.
   divEOL ^= 1;
   for (uint i = 0; i < ToolbarList.GetItemsInContainer(); i++)
      {
      TGadgetWindow* wind = ToolbarList[i];
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatDivGadget* divg = TYPESAFE_DOWNCAST(g, JPFlatDivGadget);
           if (divg)
              divg->SetEndOfLineStatus(divEOL);
           }
         }
      }
   DisposeAllToolbars();
}

void TBUTTONApp::CeDivendofline(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck(divEOL ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CmDivvertbar()
{
  // INSERT>> Your code here.
   divVertBar ^= 1;
   for (uint i = 0; i < ToolbarList.GetItemsInContainer(); i++)
      {
      TGadgetWindow* wind = ToolbarList[i];
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatDivGadget* divg = TYPESAFE_DOWNCAST(g, JPFlatDivGadget);
           if (divg)
              divg->SetNoVertBarStatus(divVertBar);
           }
         }
      }
   DisposeAllToolbars();
}


void TBUTTONApp::CeDivvertbar(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck(divVertBar ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CmToolbarcustomize()
{
  // INSERT>> Your code here.
  JPCustomizeToolbar(GetMainWindow()).Execute();
}


void TBUTTONApp::CeToolbarcustomize(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  TMenuItemEnabler* menuEnabler = TYPESAFE_DOWNCAST(&tce, TMenuItemEnabler);
  if (menuEnabler)
     UpdateToolbarMenu(menuEnabler);
}


void TBUTTONApp::CeButtonMenu(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
}

void TBUTTONApp::CeColorPicker(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
}

void TBUTTONApp::InvertOptionalParams(JPFlatButtonTextGadget::JPOptionalParams value)
{
  // INSERT>> Your code here.
   bool modified = false;
   uint maxnum = ToolbarList.GetItemsInContainer();
   TGadgetWindow* wind;
   for (uint i = 0; i <= maxnum; i++)
      {
      if (i < maxnum)
         wind = ToolbarList[i];
      else wind = GadgetAvailableList;
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatButtonTextGadget* bg = TYPESAFE_DOWNCAST(g, JPFlatButtonTextGadget);
           if (bg)
              {
              bg->InvertOptionalParams(value);
              if (!modified)
                 {
                 moreParams = bg->GetOptionalParamsValue();
                 modified = true;
                 }
              }
           }
         }
      }
  DisposeAllToolbars();
}

void TBUTTONApp::CmIEGray()
{
  // INSERT>> Your code here.
  InvertOptionalParams(JPFlatButtonTextGadget::IEGrayButtons);
}


void TBUTTONApp::CeIEGray(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((moreParams & JPFlatButtonTextGadget::IEGrayButtons) == JPFlatButtonTextGadget::IEGrayButtons);
}


void TBUTTONApp::CmGrayscale()
{
  // INSERT>> Your code here.
  InvertOptionalParams(JPFlatButtonTextGadget::GrayButtons);
}


void TBUTTONApp::CeGrayscale(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((moreParams & JPFlatButtonTextGadget::GrayButtons) == JPFlatButtonTextGadget::GrayButtons);
}


void TBUTTONApp::CmLargebuttons()
{
  // INSERT>> Your code here.
  InvertOptionalParams(JPFlatButtonTextGadget::BigButtons);
}


void TBUTTONApp::CeLargebuttons(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((moreParams & JPFlatButtonTextGadget::BigButtons) == JPFlatButtonTextGadget::BigButtons);
}


void TBUTTONApp::CmFittextwidth()
{
  // INSERT>> Your code here.
  InvertOptionalParams(JPFlatButtonTextGadget::FitToTextWidth);
}


void TBUTTONApp::CeFittextwidth(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((moreParams & JPFlatButtonTextGadget::FitToTextWidth) == JPFlatButtonTextGadget::FitToTextWidth);
}

void TBUTTONApp::CmFullofficesupport()
{
  // INSERT>> Your code here.
  InvertOptionalParams(JPFlatButtonTextGadget::FullOfficeSupport);
}


void TBUTTONApp::CeFullofficesupport(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((moreParams & JPFlatButtonTextGadget::FullOfficeSupport) == JPFlatButtonTextGadget::FullOfficeSupport);
}

void TBUTTONApp::UpdateToolbarMenu(TMenuItemEnabler* menuEnabler)
{
  HMENU mainmenu = menuEnabler->GetMenu();
  int customizeMenuPos = menuEnabler->GetPosition();
  if (mainmenu != toolbarmenu)
      {
      uint maxsize = ToolbarList.GetItemsInContainer();
      uint count = 0;
      char szBuf[255];
      for (uint cbar = 0; cbar < maxsize; cbar++)
          {
          TGadgetWindow* wind = ToolbarList[cbar];
          if (wind)
             {
             wind->GetWindowText(szBuf, sizeof(szBuf));
             uint value = wind->IsWindowVisible() ? MF_CHECKED : 0;
             bool result = (bool) ::AppendMenu(mainmenu, MF_BYPOSITION | MF_STRING | value,
                                               CM_FIRSTTOOLBARSITEM + cbar, wind->Title);
             if (result)
                {
                ::EnableMenuItem(mainmenu, CM_FIRSTTOOLBARSITEM + cbar, MF_BYCOMMAND | MF_ENABLED);
                count++;
                }
             }
          }
      if (count > 0)
         {
         ::InsertMenu(mainmenu, customizeMenuPos + 1, MF_BYPOSITION | MF_SEPARATOR, 0, NULL);
         toolbarmenu = mainmenu;
         }
      }
}


void TBUTTONApp::CmDockinggrip()
{
  // INSERT>> Your code here.
   DockingGrip ^= 1;
   if (DockingGrip)
      InsertDockingGripInAllToolbars();
   else RemoveDockingGripFromAllToolbars();
}


void TBUTTONApp::CeDockinggrip(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck(DockingGrip ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CeTogglecombobox(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck(flatcombo->IsWindowEnabled() ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CmTogglecombobox()
{
  // INSERT>> Your code here.
   flatcombo->EnableWindow(!flatcombo->IsWindowEnabled());
}

