/**************************************************************************
*  Copyright (c) 1997-1998 by Joseph Parrello. All Rights Reserved.       *
*  Site   : http://space.tin.it/computer/giparrel                         *
*  E-Mail : joparrello@geocities.com, joparrel@tin.it                     *
*                                                                         *
*  Revision:   2.0  (August 18, 1998).                                    *
*                                                                         *
*  TButtonApp application.                                                *
*                                                                         *
*  Read BUTTONAP.TXT for use and license.                                 *
*                                                                         *
*  Only for OWL 5.02 and 5.01 users.                                      *
*  OWL and ObjectWindows are trademarks of Borland International.         *
*                                                                         *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF     *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO    *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A         *
*  PARTICULAR PURPOSE.                                                    *
**************************************************************************/
#if !defined(buttonap_h)              // Sentry, use file only if it's not already included.
#define buttonap_h

#include <owl/controlb.h>
#include <owl/docking.h>
#include <owl/printer.h>
#include <owl/textgadg.h>
#include <owl/statusba.h>

#include <classlib/bags.h>
#include <classlib/arrays.h>

#include "jpdcmdif.h"
#include "jpflbute.h"
#include "jpfltdiv.h"
#include "jpfltbox.h"
#include "jpflttxt.h"
#include "jpclrpck.h"
#include "buttonmd.h"
#include "jpmrureg.h"

#include "buttonap.rh"            // Definition of all resources.

typedef struct _TBarStruct {
                int TBarData;
                TPoint pt;
                TSize size;
                } TBarStruct;

typedef struct _TBarStructEx {
                int TBarData;
                TPoint pt;
                TSize size;
                TAbsLocation loc;
                } TBarStructEx;

//{{TApplication = TBUTTONApp}}
class TBUTTONApp : public TApplication, public JPMRURegistry {
  private:
    bool            HelpState;                          // Has the help engine been used.
    bool            ContextHelp;                        // SHIFT-F1 state(context sensitive HELP).
    TCursor*        HelpCursor;                         // Context sensitive help cursor.

    void SetupSpeedBar(JPDecMDIFrame* frame, TGadgetWindow::TTileDirection direction, TFont* font);
    void AddFiles(TFileDropletList& files);
    void ProcessCmdLine(char * CmdLine);
    void RegisterInfo();
    void UnRegisterInfo();

  public:
    TBUTTONApp();
    virtual ~TBUTTONApp();

    void CreateGadgets(TDockableControlBar* cb, TFont* font = NULL, bool server = false);
    TDockableControlBar* cb;
    TDockableControlBar* custombar;
    TIArrayAsVector<TGadgetWindow> ToolbarList;
    TGadgetWindow*  GadgetAvailableList;
    JPDecMDIFrame*  frame;
    TMenu*          tbarmenu;
    HMENU           toolbarmenu;
    THarbor*        ApxHarbor;
    JPFlatTextGadget*    txtGadg;
    JPFlatColorPicker*   FlatColorPicker;
    TFont*          toolbarFont;
    JPFlatButtonTextGadget::JPDisplayType   dispType;
    JPFlatButtonTextGadget::JPTextPos       textPos;
    JPFlatButtonTextGadget::JPOptionalParams moreParams;
    int             oldCmdShow;
    uint            ToolbarRows, MaxChars;
    bool            HotTrack, DockingGrip;
    JPFlatComboBox* flatcombo;
    TGadget*        divGadget;
    bool            divEOL;
    bool            divVertBar;

    TBUTTONMDIClient* MdiClient;

    // Public data members used by the print menu commands and Paint routine in MDIChild.
    //
    TPrinter*       Printer;                           // Printer support.
    int             Printing;                          // Printing in progress.

//{{TBUTTONAppVIRTUAL_BEGIN}}
  public:
    virtual void InitMainWindow();
    virtual void InitInstance();
    virtual bool CanClose();
    virtual bool ProcessAppMsg(MSG& msg);
    /****    ADDED TO SUPPORT "LOAD AND STORE" FEATURES FOR MAIN WINDOW     */
    virtual void RetrieveMainWindowInfoFromRegistry();
    virtual void StoreMainWindowInfoOnRegistry();
    /****    ADDED TO SUPPORT "LOAD AND STORE" FEATURES FOR COLOR PICKER GADGET    ****/
    virtual void RetrieveColorPickerDataFromRegistry();
    virtual void StoreColorPickerDataOnRegistry();
    /****    ADDED TO SUPPORT MULTIPLE TOOLBARS     */
    virtual bool InsertGadgetInToolbar(int gadgid, TGadgetWindow* wind, TGadgetWindow::TPlacement place = TGadgetWindow::After, TGadget* gadgpos = 0);
    virtual bool RemoveGadgetFromToolbar(TGadget* gadg, TGadgetWindow* wind, TGadgetWindow::TPlacement place = TGadgetWindow::After, TGadget* gadgpos = 0);
    virtual void InsertSeparatorInAvailableList();
    virtual void RemoveSeparatorFromAvailableList();
    virtual void InsertDockingGripInToolbar(TGadgetWindow* wind);
    virtual void RemoveDockingGripFromToolbar(TGadgetWindow* wind);
    virtual void InsertDockingGripInAllToolbars(bool updatewindow = true);
    virtual void RemoveDockingGripFromAllToolbars(bool updatewindow = true);
    virtual TAbsLocation GetDefaultLocation(uint index);
    virtual void RetrieveSingleToolbarFromArray(int pos, TGadgetWindow* wind);
    virtual void LoadToolbarDataFromProfile();
    virtual void LoadToolbarDataFromArray(bool updatewindow = false);
    virtual void ReloadToolbarData();
    virtual void DisposeToolbarsByArray(TBarStructEx* tbarex, int tbarcount, TAbsLocation location);
    virtual void DisposeAllToolbars();
    virtual TGadget* FindGadgetInToolbars(int id);
    virtual void StoreToolbarDataInProfile();
    virtual void SetDisplayType(JPFlatButtonTextGadget::JPDisplayType disp);
    virtual void SetTextPosition(JPFlatButtonTextGadget::JPTextPos pos);
    virtual void SetTextRows(uint rows);
    virtual void SetMaxChars(uint maxchars);
    virtual void SetDivId(TGadget* id);
    virtual void InvertOptionalParams(JPFlatButtonTextGadget::JPOptionalParams value);
    virtual void UpdateToolbarMenu(TMenuItemEnabler* menuEnabler);

//{{TBUTTONAppVIRTUAL_END}}

//{{TBUTTONAppRSP_TBL_BEGIN}}
  protected:
    void EvNewView(TView& view);
    void EvCloseView(TView& view);
    void CmHelpAbout();
    void CmHelpContents();
    void CmHelpUsing();
    void EvDropFiles(TDropInfo drop);
    void EvWinIniChange(char far* section);
    void EvOwlDocument(TDocument& doc);
    int32 CmFileSelected(uint wp, int32 lp);
    int32 CmButtonMenuSelected(TParam1 param1, TParam2 param2);
    int32 CmColorPickerSelected(TParam1 param1, TParam2 param2);
    void CeToolbarbitmap(TCommandEnabler &tce);
    void CeToolbarbitmaptext(TCommandEnabler &tce);
    void CeToolbarbottom(TCommandEnabler &tce);
    void CeToolbarleft(TCommandEnabler &tce);
    void CeToolbarright(TCommandEnabler &tce);
    void CeToolbartext(TCommandEnabler &tce);
    void CeToolbartop(TCommandEnabler &tce);
    void CeOneTextRow(TCommandEnabler &tce);
    void CeTwoTextRow(TCommandEnabler &tce);
    void CmToolbarbitmap();
    void CmToolbarbitmaptext();
    void CmToolbarbottom();
    void CmToolbarleft();
    void CmToolbarright();
    void CmToolbartext();
    void CmToolbartop();
    void CmOneTextRow();
    void CmTwoTextRow();
    void CmDecreasechars();
    void CeDecreasechars(TCommandEnabler &tce);
    void CmDefaultchars();
    void CeDefaultchars(TCommandEnabler &tce);
    void CmIncreasechars();
    void CeIncreasechars(TCommandEnabler &tce);
    void CmHottracking();
    void CeHottracking(TCommandEnabler &tce);
    void CmDivbuttons();
    void CeDivbuttons(TCommandEnabler &tce);
    void CmDivcombo();
    void CeDivcombo(TCommandEnabler &tce);
    void CmDivnothing();
    void CeDivnothing(TCommandEnabler &tce);
    void CmDivendofline();
    void CeDivendofline(TCommandEnabler &tce);
    void CmDivvertbar();
    void CeDivvertbar(TCommandEnabler &tce);
    void CmToolbarcustomize();
    void CeToolbarcustomize(TCommandEnabler &tce);
    void CeButtonMenu(TCommandEnabler &tce);
    void CeColorPicker(TCommandEnabler &tce);
    void CmIEGray();
    void CeIEGray(TCommandEnabler &tce);
    void CmGrayscale();
    void CeGrayscale(TCommandEnabler &tce);
    void CmLargebuttons();
    void CeLargebuttons(TCommandEnabler &tce);
    void CmFittextwidth();
    void CeFittextwidth(TCommandEnabler &tce);
    void CmDockinggrip();
    void CeDockinggrip(TCommandEnabler &tce);
    void CeTogglecombobox(TCommandEnabler &tce);
    void CmTogglecombobox();
    void CmFullofficesupport();
    void CeFullofficesupport(TCommandEnabler &tce);
//{{TBUTTONAppRSP_TBL_END}}
DECLARE_RESPONSE_TABLE(TBUTTONApp);
};    //{{TBUTTONApp}}


#endif  // buttonap_h sentry.
