/**************************************************************************
*  Copyright (c) 1997-1998 by Joseph Parrello. All Rights Reserved.       *
*  Sites  : http://www.geocities.com/SiliconValley/Hills/8145             *
*           http://users.iol.it/joparrello                                *
*  E-Mail : joparrello@geocities.com                                      *
*                                                                         *
*  Revision:   2.1  (December 15, 1998).                                  *
*                                                                         *
*  Definition of JPFlatButtonTextGadget class.                            *
*                                                                         *
*  Read JPFLBUTE.TXT for use and license.                                 *
*                                                                         *
*  Only for OWL 5.02 and 5.01 users.                                      *
*  OWL and ObjectWindows are trademarks of Borland International.         *
*                                                                         *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF     *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO    *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A         *
*  PARTICULAR PURPOSE.                                                    *
**************************************************************************/
#include <owl/pch.h>

#if !defined(OWL_UIHELPER_H)
#include <owl/uihelper.h>
#endif
#if !defined(OWL_UIHANDLE_H)
#include <owl/uihandle.h>
#endif
#if !defined(OWL_GLYPHBTN_H)
#include <owl/glyphbtn.h>
#endif

#include "jpflbute.h"


//{{JPFlatButtonTextGadget Implementation}}

// Constants for DrawTextEx function (Win16 targets).
#if defined(BI_PLAT_WIN16)
#define DT_END_ELLIPSIS     0x00008000
#define DT_MODIFYSTRING     0x00010000
#endif

JPFlatButtonTextGadget::JPFlatButtonTextGadget (const char far* text,
                                                JPDisplayType disptype,
                                                JPTextPos    pos,
                                                uint maxChars,
                                                uint numrows,
                                                JPOptionalParams optparams,
                                                TResId glyphResIdOrIndex,
                                                int id,
                                                TType type,
                                                bool enabled,
                                                TState state,
                                                bool sharedGlyph)
:
    TButtonGadget(glyphResIdOrIndex, id, type, enabled, state, sharedGlyph)
{
  // INSERT>> Your constructor code here.
   CurrentText = NULL;
   CurrentTextLen = NULL;
   SetText(text);
   defbmcx = defbmcy = bmcx = bmcy = -1;
   DispType  = DefaultDispType  = disptype;
   TextPos   = DefaultTextPos   = pos;
   MaxChars  = DefaultMaxChars  = maxChars;
   NumRows   = DefaultNumRows   = max(1, numrows);
   OptParams = DefaultOptParams = optparams;
   CheckOptionalParams();
   RetrieveHotTrackInfo();
   StoredBorders = GetBorders();
   if (DispType == BitmapText)
      {
      TBorders bord;
      bord.Left = bord.Right = bord.Top = bord.Bottom = 1;
      Borders = bord;
      }
   isEnter = isStoredRect = false;
   if (State==Down)
   	SetFlatState(fsDown);
   else SetFlatState(fsNormal);
}


JPFlatButtonTextGadget::~JPFlatButtonTextGadget()
{
  // INSERT>> Your destructor code here.
  delete[] CurrentText;
}


void JPFlatButtonTextGadget::PaintBorder(TDC& dc)
{
   if (GetFlatState() == fsUp)
      {
      TRect rect(0,0,Bounds.Width(),Bounds.Height());
      #if defined(BI_PLAT_WIN16)
      TUIBorder border(rect, TUIBorder::TEdge(TUIBorder::WndRaised), TUIBorder::Rect | TUIBorder::Adjust);
      #elif defined(BI_PLAT_WIN32)
      TUIBorder border(rect, TUIBorder::TEdge(TUIBorder::RaisedInner), TUIBorder::Rect | TUIBorder::Adjust);
      #endif
      border.Paint(dc);
      }
   else if (GetFlatState() == fsDown)
      {
      BorderStyle = Recessed;
      TButtonGadget::PaintBorder(dc);
      }
}

void
JPFlatButtonTextGadget::SetFlatState(FlatState state, bool update)
{
switch (state)
	{
   case fsNormal:
      CurrentFlatState = fsNormal;
      break;

   case fsUp:
      if (GetEnabled())
         CurrentFlatState = fsUp;
      else return;
      break;

   case fsDown:
      if ((GetEnabled()) || (GetButtonState() == Down))
         CurrentFlatState = fsDown;
      else return;
      break;
   }
if (update)
	{
   if (DispType == Text)
      InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
   else
      InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), false);
	Update();
   }
}

JPFlatButtonTextGadget::FlatState
JPFlatButtonTextGadget::GetFlatState() const
{
  return  CurrentFlatState;
}


void JPFlatButtonTextGadget::MouseEnter(uint modKeys, TPoint& point)
{
  TButtonGadget::MouseEnter(modKeys, point);

  // INSERT>> Your code here.
  isEnter = true;
  if ((GetButtonState()==Up) && (GetEnabled()))
      {
     if (DispType == Text)
        InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), false);
  		SetFlatState(fsUp);
      }
  else if (GetFlatState()==fsDown)
          InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), GetOptionalParams(FullOfficeSupport));
}


void JPFlatButtonTextGadget::MouseLeave(uint modKeys, TPoint& point)
{
  TButtonGadget::MouseLeave(modKeys, point);

  // INSERT>> Your code here.
  isEnter = false;
  if ((State!=Down) && (GetEnabled()))
     {
     TSize size;
     GetDefaultBitmapSize(size);
     if ((DispType == Text) || (size.cx == 0) || (size.cy == 0))
        InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
     SetFlatState(fsNormal);
     }
  else if (GetFlatState()==fsDown)
          InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), GetOptionalParams(FullOfficeSupport));
}


void JPFlatButtonTextGadget::BeginPressed(TPoint& p)
{
  SetFlatState(fsDown , false);
  if (DispType != Bitmap)
     InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
  if (isEnter == false)
     {
     isEnter = Pressed = true;
     TButtonGadget::MouseEnter(0, p);
     }
  TButtonGadget::BeginPressed(p);
}


void JPFlatButtonTextGadget::CancelPressed(TPoint& p)
{
  bool oldpressed = Pressed;
  if (DispType != Bitmap)
     InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
  if ((State==Up) && (oldpressed))
  		{
      bool hit = PtIn(p);
      if ((hit) && GetEnabled())
  				SetFlatState(fsUp, false);
      else
         {
         isEnter = Pressed = false;
  			SetFlatState(fsNormal);
         TButtonGadget::MouseLeave(0, p);
         }
      }
  TButtonGadget::CancelPressed(p);
}


void
JPFlatButtonTextGadget::SetButtonState(TState newState)
{
  TButtonGadget::SetButtonState(newState);
   if (State==Down)
      SetFlatState(fsDown);
   else
      SetFlatState(fsNormal);
}

bool
JPFlatButtonTextGadget::IdleAction(long idleCount)
{
  if (idleCount == 0)
    {
    if (GetEnabled()==false)
    	{
    	if ((GetFlatState()!=fsNormal) && (GetButtonState()==Up) && (GetButtonType()==Command))
    	   SetFlatState(fsNormal);
      }
    else
    	{
      if (isEnter==true)
      	{
      	if ((GetFlatState()==fsNormal) && (GetButtonState()==Up))
            SetFlatState(fsUp);
         else if ((GetFlatState()==fsUp) && (GetButtonState()==Down))
         	SetFlatState(fsDown);
         }
      }
    }
  return TButtonGadget::IdleAction(idleCount);
}

void
JPFlatButtonTextGadget::SetEnabled(bool enabled)
{
  SetTextFromResource();        // retrieve the text string when possible
  if ((enabled != GetEnabled()) && (DispType == Text))
     {
     if ((!enabled) && (GetFlatState() == fsUp))
        SetFlatState(fsNormal,false);
     InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true); // erase shadow of text
     }
  TButtonGadget::SetEnabled(enabled);
}

/*******************************************************************************
*   Called when system colors change.                                          *
*   Under Windows 98 retrieve the system's hot-track colors (if available).    *
*******************************************************************************/
void
JPFlatButtonTextGadget::SysColorChange()
{
  TButtonGadget::SysColorChange();

  RetrieveHotTrackInfo();
  InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
  Update();
}

/*******************************************************************************
*   This method returns the size of the simple gadget if bitmap isn't needed.  *
*******************************************************************************/
void JPFlatButtonTextGadget::GetDesiredSize(TSize& size)
{
  if (DispType == Text)
     TGadget::GetDesiredSize(size);
  else
     {
     TGadget::GetDesiredSize(size);
     TSize bitmapsize;
     GetCurrentBitmapSize(bitmapsize);
     size.cx = size.cx + bitmapsize.cx;
     size.cy = size.cy + bitmapsize.cy;
     }
}

/*******************************************************************************
*   This method paints the image before and the text after.                    *
*******************************************************************************/
void JPFlatButtonTextGadget::Paint(TDC& dc)
{
    uint fstate = GetFlatState();
    uint nstate = GetButtonState();
    if ((fstate != fsUp) && (!Pressed))
    	{
      if ((nstate == Down) && (fstate != fsDown))
      	{
         SetFlatState(fsDown, false);
         InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
         }
      else if ((nstate == Up) && (fstate != fsNormal))
         {
         SetFlatState(fsNormal, false);
   		InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
         }
      }

  // We draw the button image.
  DrawButtonImage(dc);

  // We draw the text now.
  if (DispType != Bitmap)
     {
     if (Window)
        {
        TFont* font = &(Window->GetFont());
        if (font)
           dc.SelectObject(*font);
        }
     TRect rect1;
     rect1 = CurrentTextRect;
     uint offset = 0;
     if (GetOptionalParams(FullOfficeSupport))
        {
        if (Pressed && State == Down)
           offset = 2;
        else if (State == Down || Pressed)
                offset = 1;
        }
     else
        offset = (Pressed || State == Down) ? 1 : 0;
     rect1.Offset(offset, offset);
     int oldbkmode = dc.SetBkMode(TRANSPARENT);
     TColor oldtextcolor = dc.GetTextColor();
     if (GetEnabled() == false)
        {
        rect1.Offset(1,1);
        dc.SetTextColor(TColor::Sys3dHilight);
        if (NumRows > 1)
           DrawTextEx(dc, CurrentText, CurrentTextLen, rect1, DT_CENTER | DT_END_ELLIPSIS | DT_WORDBREAK);
        else
           DrawTextEx(dc, CurrentText, CurrentTextLen, rect1, DT_LEFT | DT_SINGLELINE | DT_END_ELLIPSIS);
        rect1.Offset(-1,-1);
        }
     if ((GetFlatState()==fsNormal) || (HotTrack == false) || (isEnter == false))
        dc.SetTextColor(GetEnabled() ? TColor::SysBtnText : TColor::SysGrayText);
     else
        dc.SetTextColor(GetEnabled() ? TColor(GetHotTrackColor()) : TColor::SysGrayText);
     if (NumRows > 1)
         DrawTextEx(dc, CurrentText, CurrentTextLen, rect1, DT_CENTER | DT_END_ELLIPSIS | DT_WORDBREAK);
     else
         DrawTextEx(dc, CurrentText, CurrentTextLen, rect1, DT_LEFT | DT_SINGLELINE | DT_END_ELLIPSIS);
     dc.SetBkMode(oldbkmode);
     dc.SetTextColor(oldtextcolor);
     dc.RestoreFont();
   }
}

/*******************************************************************************
*   This method draws the button image.                                        *
*******************************************************************************/
void JPFlatButtonTextGadget::DrawButtonImage(TDC& dc)
{
   if (DispType != Text)
      {
      // Retrieve the image dib.
      TDib* Dib = GetImageDib();
      if (Dib)
         {
         bool value = (((isEnter == false) && (GetOptionalParams(IEGrayButtons))) ||
                         GetOptionalParams(GrayButtons)) && GetEnabled();
         if (!value)
            Dib->MapUIColors( TDib::MapFace | TDib::MapText | TDib::MapShadow | TDib::MapHighlight );

         // Retrieve the dib info.
         BITMAPINFO &bmInfo = *(LPBITMAPINFO) *Dib ;

         int nColors = (int) Dib->NumColors();

         // Convert DIB to grey-scale if necessary.
         if (value && (nColors <= 256))
            {
            for( int i=0; i < nColors; i++)
               {
               if (!((bmInfo.bmiColors[i].rgbRed == bmInfo.bmiColors[i].rgbGreen) &&
                   (bmInfo.bmiColors[i].rgbRed == bmInfo.bmiColors[i].rgbBlue)))
                  {
                  BYTE nGray = (BYTE) ((bmInfo.bmiColors[i].rgbGreen +
                                       (bmInfo.bmiColors[i].rgbRed / 2) +
                                       (bmInfo.bmiColors[i].rgbBlue / 2)) / 2);
                  bmInfo.bmiColors[i].rgbRed = nGray;
                  bmInfo.bmiColors[i].rgbGreen = nGray;
                  bmInfo.bmiColors[i].rgbBlue = nGray;
                  }
               }
            Dib->MapUIColors( TDib::MapFace | TDib::MapText | TDib::MapShadow | TDib::MapHighlight );
            }

         // We increase the size of the button image if necessary and we draw the image.
         LPVOID lpDIBBits = (LPVOID)(bmInfo.bmiColors + nColors);
         TRect rect;
         GetInnerRect(rect);
         TBitmap* startBitmap = new TBitmap(*Dib);
         BITMAP bm1, bm2;
         startBitmap->GetObject(bm1);
         bm2 = bm1;
         bm2.bmWidth = rect.Width();
         bm2.bmHeight = rect.Height();
         bm2.bmWidthBytes = ((bm2.bmWidth * bm2.bmBitsPixel + 15) / 16) * 2;
         TBitmap* faceBitmap = new TBitmap(&bm2);
         TMemoryDC* hdcMem2 = new TMemoryDC(*faceBitmap);
         ::StretchDIBits(hdcMem2->GetHDC(), 0, 0,	rect.Width(), rect.Height(), 0,	0,
                              Dib->Width(), Dib->Height(), lpDIBBits, (LPBITMAPINFO) *Dib,
                              DIB_RGB_COLORS, SRCCOPY);
         delete hdcMem2;
         delete startBitmap;
         TRect boundrect = GetBounds();
         TRect destrect(0,0, boundrect.Width(), boundrect.Height());
         TPoint pt (rect.left, rect.top);
         TUIFace face(destrect, *faceBitmap);
         if (!GetEnabled())
            face.Paint(dc, pt, TUIFace::Disabled, false);
         else if (State == Indeterminate)
            face.Paint(dc, pt, TUIFace::Indeterm, Pressed);
         /*else if (State == Down && !Pressed)
            face.Paint(dc, pt, TUIFace::Down, false);*/
         else if (State == Down)
                 {
                 if (GetOptionalParams(FullOfficeSupport))
                     face.Paint(dc, isEnter ? pt+TPoint(1,1) : pt, isEnter ? TUIFace::Normal : TUIFace::Down, Pressed);
                 else
                     face.Paint(dc, pt, Pressed ? TUIFace::Normal : TUIFace::Down, Pressed);
                 }
         else
            face.Paint(dc, pt, TUIFace::Normal, Pressed);
         delete faceBitmap;
         delete Dib;
         }
      }
   else
      {
      // If the button is only text and it's pressed or down, we must fill
      // its area with a Hatched Brush.
      if (GetFlatState()==fsDown && ((GetOptionalParams(FullOfficeSupport)==true && isEnter==false) ||
                                     (GetOptionalParams(FullOfficeSupport)==false && Pressed==false)))
         {
         TRect rect;
         GetInnerRect(rect);
         TRect boundrect = GetBounds();
         TRect destrect(0,0, boundrect.Width(), boundrect.Height());
         TUIFace face(destrect, NULL);
         THatch8x8Brush brush = face.GetDitherBrush();
         dc.SelectObject(brush);
         dc.Rectangle(destrect);
         dc.RestoreBrush();
         }
      }

   // We paint the border for this button.
   BorderStyle = (Pressed || State == Down) ? ButtonDn : ButtonUp;
   PaintBorder(dc);
}

/*******************************************************************************
*   This method sets the current text.                                         *
*******************************************************************************/
void JPFlatButtonTextGadget::SetText(const char far* text)
{
  if (text && CurrentText && strcmp(text, CurrentText) == 0)
    return;

  delete[] CurrentText;

  if (text)
    {
    CurrentText = strnewdup(text);
    CurrentTextLen = strlen(CurrentText);
    }
  else
    {
    if (SetTextFromResource() == false)
       {
       CurrentText = NULL;
       CurrentTextLen = NULL;
       }
    }

  if (Window)
    Window->GadgetChangedSize(*this);

  Invalidate();
}

/*******************************************************************************
*   This method retrieves the current text.                                    *
*******************************************************************************/
const char far* JPFlatButtonTextGadget::GetText() const
{
  return CurrentText;
}

/*******************************************************************************
*   This method tries to retrieve the resource string as the text.             *
*******************************************************************************/
bool JPFlatButtonTextGadget::SetTextFromResource()
{
    if ((Window) && (Window->GetHandle()) && (!CurrentText))
       {
       string text = LoadInternalString(GetId());
       if (text.length())
          {
          delete [] CurrentText;
          CurrentText = strnewdup(text.c_str());
          CurrentTextLen = strlen(CurrentText);
          return true;
          }
       }
    return false;
}

/*******************************************************************************
*   This method is called only when button has been created.                   *
*******************************************************************************/
void JPFlatButtonTextGadget::Created()
{
    TButtonGadget::Created();

    if (!isStoredRect)
       {
       // store the default size of the bitmap (used by other methods).
       TSize size;
       GetCurrentBitmapSize(size);
       StoredRect = TRect(TPoint(0,0),size);
       isStoredRect = true;
       }

    // if there is no text, tries to retrieve the resource string.
    SetTextFromResource();

    // dispose the bitmap and the text on the gadget.
    DisposeUnitsOnGadget();
}

/*******************************************************************************
*   This method disposes the bitmap and the text on the gadget.                *
*******************************************************************************/
void JPFlatButtonTextGadget::DisposeUnitsOnGadget()
{
  if ((Window) && (Window->GetHandle()))
      {
      TClientDC dc(Window->GetHandle());
      TFont* font = &(Window->GetFont());
      if (font)
          dc.SelectObject(*font);
      int MaxWidth  = (MaxChars * font->GetMaxWidth());
      int maxwidth  = MaxWidth;
      TEXTMETRIC tm;
      font->GetTextMetrics(tm);
      int MaxHeight = tm.tmHeight + tm.tmExternalLeading;
      int numrow = MaxHeight * NumRows;
      TRect rect(0, 0, MaxWidth, numrow);
      if (NumRows > 1)
         dc.DrawText(CurrentText, -1, rect, DT_CALCRECT | DT_WORDBREAK);
      else
         dc.DrawText(CurrentText, -1, rect, DT_CALCRECT | DT_SINGLELINE);
      dc.RestoreFont();
      CurrentTextRect = rect;
      if (((TextPos == Left) || (TextPos == Right)) && (DispType == BitmapText))
         {
         if (numrow < StoredRect.Height())
            rect.bottom = 0;
         else rect.bottom = numrow - StoredRect.Height();
         }
      else
         rect.bottom = numrow;
      if ((CurrentTextRect.right > MaxWidth) && (!GetOptionalParams(FitToTextWidth)))
         CurrentTextRect.right = MaxWidth;
      if (CurrentTextRect.bottom > numrow)
         CurrentTextRect.bottom = numrow;
      if (GetOptionalParams(FitToTextWidth) && (strlen(CurrentText) > 0))
         MaxWidth = rect.right + 1;
      else
         MaxWidth = maxwidth;

      if (DispType != Bitmap)
         {
         if (((TextPos == Top) || (TextPos == Bottom)) && (DispType == BitmapText))
            if (MaxWidth > StoredRect.Width())
               MaxWidth = MaxWidth - StoredRect.Width();
            else
                if (GetOptionalParams(FitToTextWidth))
                   MaxWidth = 0;
                else
                   MaxWidth = StoredRect.Width();
         TMargins mrect;
         mrect = Margins;
         mrect.Units = TMargins::Pixels;
         TBorders bord;
         if (DispType == BitmapText)
            bord.Left = bord.Right = bord.Top = bord.Bottom = 1;
         else bord = StoredBorders;
         Borders = bord;

         if (DispType == BitmapText)
            {
            switch (TextPos) {
             case Left:
               mrect.Left   = MaxWidth + bord.Left;
               mrect.Top    = rect.bottom / 2 + bord.Top;
               mrect.Bottom = rect.bottom / 2 + bord.Bottom;
               mrect.Right = 0;
               break;

             case Right:
               mrect.Right = MaxWidth + bord.Right;
               mrect.Top    = rect.bottom / 2 + bord.Top;
               mrect.Bottom = rect.bottom / 2 + bord.Bottom;
               mrect.Left = 0;
               break;

             case Top:
               mrect.Left = mrect.Right = MaxWidth / 2 + bord.Left;
               mrect.Top = rect.bottom + bord.Bottom + bord.Top;
               mrect.Bottom = 0;
               break;

             case Bottom:
               mrect.Left = mrect.Right = MaxWidth / 2 + bord.Left;
               mrect.Bottom = rect.bottom + bord.Bottom + bord.Top;
               mrect.Top = 0;
               break;
             }
            }
            else
               {
               mrect.Left = mrect.Right = (MaxWidth / 2) + bord.Left;
               mrect.Bottom = rect.bottom + bord.Bottom + bord.Top;
               mrect.Top = 0;
               }

         SetMargins(mrect);
         TRect  rectBounds = Bounds;
         SetBounds(rectBounds);
         TSize sz;
         Window->GetDesiredSize(sz);

         // If the parent window is a TGadgetWindow, we must resize its client area
         // (we don't need this code if the parent window is a Dockable Window).
         if ((Window->Attr.W != sz.cx) || (Window->Attr.H != sz.cy))
           {
           Window->Attr.W = sz.cx;
           Window->Attr.H = sz.cy;
           TWindow* parent= Window->GetParentO();
           TDecoratedFrame* frame= parent ?
                                   TYPESAFE_DOWNCAST(parent, TDecoratedFrame) : 0;
           while (parent && !frame) {
             parent = parent->GetParentO();
             if (parent)
               frame = TYPESAFE_DOWNCAST(parent, TDecoratedFrame);
             }
           if (frame)
              {
              TLayoutMetrics  metrics;
              frame->GetChildLayoutMetrics(*Window, metrics);
              frame->SetChildLayoutMetrics(*Window, metrics);
              frame->Layout();
              }
           }
         }
     TRect rect1;
     GetInnerRect(rect);
     rect1 = CurrentTextRect;
     TBorders border = GetBorders();
     int numwidth = (DispType == Text) ? 0 : StoredRect.Width();
     if (border.Left == border.Right)
        rect1.Offset(border.Left, 0);
     if (rect1.Width() >= Bounds.Width())
         rect1.right = rect1.left + Bounds.Width() - 1;

     if (DispType != Text)
        {
        switch (TextPos) {
          case Left:
            if ( (rect1.Width() + numwidth) < Bounds.Width() )
               rect1.Offset( (Bounds.Width() - numwidth - rect1.Width()) / 2, 0);
            rect1.Offset(0, (Bounds.Height() - rect1.Height()) / 2);
            break;

          case Right:
            if ( (rect1.Width() + numwidth) < Bounds.Width() )
               rect1.Offset(numwidth + (Bounds.Width() - numwidth - rect1.Width() - 1) / 2,0);
            rect1.Offset(0, (Bounds.Height() - rect1.Height()) / 2);
            break;

          case Top:
            if (rect1.Width() < Bounds.Width())
               rect1.Offset(((Bounds.Width() - rect1.Width() - 1) / 2), 0);
            rect1.Offset(0, border.Top + ((rect.top - rect1.Height()) / 2));
            break;

          case Bottom:
            if (rect1.Width() < Bounds.Width())
               rect1.Offset(((Bounds.Width() - rect1.Width() - 1) / 2), 0);
            rect1.Offset(0, rect.bottom + ((Bounds.Height()-rect.bottom-rect1.Height()-1) / 2));
            break;
          }
        }
     else
        {
         if (rect1.Width() < Bounds.Width())
            rect1.Offset(((Bounds.Width() - rect1.Width() - 1) / 2), 0);
         rect1.Offset(0, ((Bounds.Height()-rect.bottom-rect1.Height()) / 2));
        }
     CurrentTextRect = rect1;
     }
}

/*******************************************************************************
*   This method allows to retrieve the resource string for this button.        *
*******************************************************************************/
string JPFlatButtonTextGadget::LoadInternalString(uint itemId)
{
   char tempstr[256], szBuf[256];
   char* ptr = NULL, c = '\n';
   strcpy(szBuf,"");
   strcpy(tempstr,"");
   int len = Window->GetModule()->LoadString(itemId, tempstr, sizeof(tempstr));
   if (len > 0)
      ptr = strchr(tempstr, c);
   if (ptr)
      strncpy(szBuf, ptr + 1, min(len, (int) sizeof(szBuf)));
   else
      {
      TWindow* parent= Window->GetParentO();
      TDecoratedFrame* frame= parent ? TYPESAFE_DOWNCAST(parent, TDecoratedFrame) : 0;
      while (parent && !frame)
      {
        parent = parent->GetParentO();
        if (parent)
          frame = TYPESAFE_DOWNCAST(parent, TDecoratedFrame);
      }
      if (frame)
          frame->GetHintText(itemId, szBuf, sizeof(szBuf), htTooltip);
      else if (len > 0)
              strcpy(szBuf, tempstr);
      }
   return string(szBuf);
}

/*******************************************************************************
*   This method allows the use of DrawTextEx (not defined by OWL).             *
*******************************************************************************/
#if defined(BI_PLAT_WIN32)
int
JPFlatButtonTextGadget::DrawTextEx(TDC& dc, const char far* str, int count,
                                   const TRect& rect, uint16 format)
{
  DRAWTEXTPARAMS param;
  param.cbSize = sizeof (DRAWTEXTPARAMS);
  param.iTabLength = param.iLeftMargin = param.iRightMargin = 0;
  param.uiLengthDrawn = count;
  return ::DrawTextEx(dc, (LPTSTR) str, count, (RECT*)&rect, format, &param);
}
/*******************************************************************************
*   This method emulates DrawTextEx on Win16 platforms.                        *
*   It emulates DT_END_ELLIPSIS flag too, but is limited to draw.              *
*   Don't use this method with DT_CALCRECT or DT_MODIFYSTRING.                 *
*******************************************************************************/
#elif defined(BI_PLAT_WIN16)
int
JPFlatButtonTextGadget::DrawTextEx(TDC& dc, const char far* str, int count,
                                   const TRect& rect, uint16 format)
{
   int retValue = 0;
   bool isDrawn = false;
   bool win32Bit = true;
   DWORD dwVersion = ::GetVersion();
   if ((LOBYTE(LOWORD(dwVersion)) == 3) && (HIBYTE(LOWORD(dwVersion)) < 95))
      win32Bit = false;
  if (((format & DT_END_ELLIPSIS) == DT_END_ELLIPSIS) && (win32Bit == false))
     {
     format &= ~DT_END_ELLIPSIS;
     TRect newrect = rect;
     dc.DrawText(str, count, newrect, DT_CALCRECT | format);
     if (rect.Contains(newrect) == false)
        {
        char buf[256];
        char *ellipse = "...";
        int buflen;
        for (int i = strlen(str); i--; i == 0)
           {
           strncpy(buf, str, i + 1);
           buf[i + 1] = '\0';
           strcat(buf, ellipse);
           buflen = strlen(buf);
           newrect = rect;
           dc.DrawText(buf, buflen, newrect, DT_CALCRECT | format);
           if (rect.Contains(newrect) == true)
               {
               retValue = dc.DrawText(buf, buflen, rect, format);
               isDrawn = true;
               break;
               }
           }
        }
     }
  if (isDrawn == false)
     retValue = dc.DrawText(str, count, rect, format);
  return retValue;
}
#endif

/*******************************************************************************
*   This method retrieve informations about Hot-Tracking feature.              *
*******************************************************************************/
void
JPFlatButtonTextGadget::RetrieveHotTrackInfo()
{
#ifndef COLOR_HOTLIGHT
#define COLOR_HOTLIGHT  26
#endif
#ifndef SPI_GETHOTTRACKING
#define SPI_GETHOTTRACKING 0x100E
#endif
   bool result = (bool) ::SystemParametersInfo(SPI_GETHOTTRACKING, NULL, &HotTrack, NULL);
   if (result)
      {
      HotTrackColor = TColor(::GetSysColor(COLOR_HOTLIGHT));
      Invalidate();
      Update();
      }
   else
      {
      HotTrack = true;               // Hot-Tracking is enabled by default.
      SetHotTrackColor(0, 0, 255);   // Default Hottrack color is Blue.
      }
}

/*******************************************************************************
*   This method sets the actual Hot Track color for the text.                  *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetHotTrackColor(uint8 r, uint8 g, uint8 b)
{
    HotTrackColor = TRgbTriple(r,g,b);

    Invalidate();
    Update();
}

/*******************************************************************************
*   This method returns the actual Hot Track color for the text.               *
*******************************************************************************/
COLORREF
JPFlatButtonTextGadget::GetHotTrackColor()
{
  return HotTrackColor.GetValue();
}

/*******************************************************************************
*   This method sets the actual boolean Hot Track value for the text.          *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetHotTrack(bool track)
{
    HotTrack = track;

    Invalidate();
    Update();
}

/*******************************************************************************
*   This method returns the actual boolean Hot Track value for the text.       *
*******************************************************************************/
bool
JPFlatButtonTextGadget::GetHotTrack()
{
  return HotTrack;
}

/*******************************************************************************
*    Set the display type for this button (Text, Bitmap, BitmapText).          *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetDisplayType(JPDisplayType disp)
{
  if (disp != DispType)
     {
     DispType = disp;
     DisposeUnitsOnGadget();
     if (DispType == Bitmap)
        {
        Borders = StoredBorders;
        TMargins mrect;
        SetMargins(mrect);
        }
     Invalidate();
     }
}

/*******************************************************************************
*    Returns the display type for this button (Text, Bitmap, BitmapText).      *
*******************************************************************************/
JPFlatButtonTextGadget::JPDisplayType
JPFlatButtonTextGadget::GetDisplayType() const
{
  return  DispType;
}

/*******************************************************************************
*    Set the text position for this button (Left, Right, Top, Bottom).         *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetTextPosition(JPTextPos pos)
{
  if (pos != TextPos)
     {
     TextPos = pos;
     DisposeUnitsOnGadget();
     if (DispType == Text)
        InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
     else
        InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), false);
     Update();
     }
}

/*******************************************************************************
*    Returns the text position for this button (Left, Right, Top, Bottom).     *
*******************************************************************************/
JPFlatButtonTextGadget::JPTextPos
JPFlatButtonTextGadget::GetTextPosition() const
{
  return  TextPos;
}

/*******************************************************************************
*    Set the text number of rows for this button.                              *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetTextRows(uint rows)
{
  if (rows != NumRows)
     {
     NumRows = max(1, rows);
     InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
     DisposeUnitsOnGadget();
     Update();
     }
}

/*******************************************************************************
*    Returns the text number of rows for this button.                          *
*******************************************************************************/
uint
JPFlatButtonTextGadget::GetTextRows() const
{
  return  NumRows;
}

/*******************************************************************************
*    Set the max text number of chars for this button.                         *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetMaxChars(uint maxchars)
{
  if (maxchars != MaxChars)
     {
     MaxChars = maxchars;
     DisposeUnitsOnGadget();
     Update();
     }
}

/*******************************************************************************
*    Returns the max text number of chars for this button.                     *
*******************************************************************************/
uint
JPFlatButtonTextGadget::GetMaxChars() const
{
  return  MaxChars;
}

/*******************************************************************************
*    Set the current status of an optional parameter.                          *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetOptionalParams(JPOptionalParams value, bool enable, bool update)
{
#pragma warn -eas
  if (enable != GetOptionalParams(value))
     {
     if (enable)
        OptParams = OptParams | value;
     else OptParams &= ~value;
     if ((value == IEGrayButtons) && (enable))
        OptParams &= ~GrayButtons;
     CheckOptionalParams();
     if (value == BigButtons)
        {
        defbmcx = defbmcy = bmcx = bmcy = -1;
        TSize size;
        GetCurrentBitmapSize(size);
        StoredRect = TRect(TPoint(0,0),size);
        }
     if (update)
        {
        InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
        DisposeUnitsOnGadget();
        if (DispType == Bitmap)
           {
           Borders = StoredBorders;
           TMargins mrect;
           SetMargins(mrect);
           }
        Update();
        }
     }
#pragma warn .eas
}

/*******************************************************************************
*    Check the current status of optional parameters.                          *
*******************************************************************************/
void
JPFlatButtonTextGadget::CheckOptionalParams()
{
#pragma warn -eas
   if ((GetOptionalParams(IEGrayButtons)) && (GetOptionalParams(GrayButtons)))
      OptParams &= ~IEGrayButtons;
   if (GetOptionalParams(FitToTextWidth))
      NumRows = 1;
#pragma warn .eas
}

/*******************************************************************************
*    Retrieve the current status of an optional parameter.                     *
*******************************************************************************/
bool
JPFlatButtonTextGadget::GetOptionalParams(JPOptionalParams value)
{
  return (bool) ((OptParams & value) == value);
}

/*******************************************************************************
*    Retrieve the current status of an optional parameter.                     *
*******************************************************************************/
JPFlatButtonTextGadget::JPOptionalParams
JPFlatButtonTextGadget::GetOptionalParamsValue()
{
  return OptParams;
}

/*******************************************************************************
*    Invert the current status of an optional parameter.                       *
*******************************************************************************/
void
JPFlatButtonTextGadget::InvertOptionalParams(JPOptionalParams value)
{
  bool enable = GetOptionalParams(value) ^ 1;
  SetOptionalParams(value, enable);
}

/*******************************************************************************
*    Set the default display type (as declared on constructor).                *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetDefaultDisplayType()
{
   SetDisplayType(DefaultDispType);
}

/*******************************************************************************
*    Set the default text position (as declared on constructor).               *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetDefaultTextPosition()
{
   SetTextPosition(DefaultTextPos);
}

/*******************************************************************************
*    Set the default text number of rows (as declared on constructor).         *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetDefaultTextRows()
{
   SetTextRows(DefaultNumRows);
}

/*******************************************************************************
*    Set the default max text number of chars (as declared on constructor).    *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetDefaultMaxChars()
{
   SetMaxChars(DefaultMaxChars);
}

/*******************************************************************************
*    Set the default optional parameters (as declared on constructor).         *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetDefaultOptionalParameters()
{
   OptParams = DefaultOptParams;
   DisposeUnitsOnGadget();
   Update();
}

/*******************************************************************************
*   This method returns the current size of the bitmap image.                  *
*******************************************************************************/
void JPFlatButtonTextGadget::GetCurrentBitmapSize(TSize& size)
{
   if (bmcx == (int) -1)
      {
      TDib* dib = GetGlyphDib();
      if (dib)
         {
         size.cx = dib->Width();
         size.cy = dib->Height();
         }
      else size = (0,0);
      defbmcx = size.cx;
      defbmcy = size.cy;
      if (GetOptionalParams(BigButtons))
         {
         size.cx = size.cx * 2;
         size.cy = size.cy * 2;
         }
      bmcx = size.cx;
      bmcy = size.cy;
      delete dib;
      }
   else
      {
      size.cx = bmcx;
      size.cy = bmcy;
      }
}

/*******************************************************************************
*   This method returns the size of the default bitmap.                        *
*******************************************************************************/
void JPFlatButtonTextGadget::GetDefaultBitmapSize(TSize& size)
{
   if (defbmcx == (int) -1)
      {
      TDib* dib = GetGlyphDib();
      size.cx = dib->Width();
      size.cy = dib->Height();
      defbmcx = size.cx;
      defbmcy = size.cy;
      delete dib;
      }
   else
      {
      size.cx = defbmcx;
      size.cy = defbmcy;
      }
}

/*******************************************************************************
*   This method returns the default glyph dib.                                 *
*******************************************************************************/
TDib* JPFlatButtonTextGadget::GetDefaultGlyphDib()
{
   return GetGlyphDib();
}

/*******************************************************************************
*   This method returns the current image dib.                                 *
*******************************************************************************/
TDib* JPFlatButtonTextGadget::GetImageDib()
{
  if (ResId) {
    TDib* glyph = new TDib(*Window->GetModule(), ResId);
    return glyph;
  }
  return 0;
}

