/**************************************************************************
*  Copyright (c) 1997-1998 by Joseph Parrello. All Rights Reserved.       *
*  Sites  : http://www.geocities.com/SiliconValley/Hills/8145             *
*           http://users.iol.it/joparrello                                *
*  E-Mail : joparrello@geocities.com                                      *
*                                                                         *
*  Revision:   2.1  (December 15, 1998).                                  *
*                                                                         *
*  Definition of JPFlatButtonTextGadget class.                            *
*                                                                         *
*  Read JPFLBUTE.TXT for use and license.                                 *
*                                                                         *
*  Only for OWL 5.02 and 5.01 users.                                      *
*  OWL and ObjectWindows are trademarks of Borland International.         *
*                                                                         *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF     *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO    *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A         *
*  PARTICULAR PURPOSE.                                                    *
**************************************************************************/
#if !defined(jpfltbtg_h)              // Sentry, use file only if it's not already included.
#define jpfltbtg_h

#if !defined(OWL_BUTTONGA_H)
#include <owl/buttonga.h>
#endif
#if !defined(OWL_GADGETWI_H)
#include <owl/gadgetwi.h>
#endif

//{{TButtonGadget = JPFlatButtonTextGadget}}
class JPFlatButtonTextGadget : public TButtonGadget {
  public:
    enum JPDisplayType {
         Bitmap,
         Text,
         BitmapText
         };

    enum JPTextPos {
         Left,
         Right,
         Top,
         Bottom
         };

    enum JPOptionalParams {
         IEGrayButtons     = 0x01,
         GrayButtons       = 0x02,
         BigButtons        = 0x04,
         FitToTextWidth    = 0x08,
         FullOfficeSupport = 0x10
         };

    JPFlatButtonTextGadget(const char far* text,
                           JPDisplayType disptype,
                           JPTextPos    pos,
                           uint maxChars,
                           uint numrows,
                           JPOptionalParams optparams,
                           TResId glyphResIdOrIndex,
                           int id,
                           TType type = Command,
                           bool enabled = false,
                           TState state = Up,
                           bool sharedGlyph = false);
    virtual ~JPFlatButtonTextGadget();

//{{JPFlatButtonTextGadgetVIRTUAL_BEGIN}}
  protected_data:
    enum          FlatState {fsNormal, fsUp, fsDown};
    char far*     CurrentText;            // new'd copy of the text for this gadget
    uint          CurrentTextLen;         // strlen of the above text
    TRect         CurrentTextRect;        // Rectangle where the text will appear
    TRect         StoredRect;             // the bitmap's rect
    TColor        HotTrackColor;          // Hot Track Color for Text
    bool          HotTrack;               // Enable/Disable Hot Track for Text
    TBorders      StoredBorders;
    JPDisplayType     DispType,  DefaultDispType;
    JPTextPos         TextPos,   DefaultTextPos;
    JPOptionalParams  OptParams, DefaultOptParams;
    uint          MaxChars, DefaultMaxChars;
    uint          NumRows,  DefaultNumRows;
    FlatState     CurrentFlatState;
    bool          isEnter, isStoredRect;
    int           defbmcx, defbmcy, bmcx, bmcy;

  protected:
	 void 			SetFlatState(FlatState state, bool update=true);
    FlatState     GetFlatState() const;
    void          Paint(TDC& dc);
    void          Created();
    void          DisposeUnitsOnGadget();
    bool          SetTextFromResource();
    int           DrawTextEx(TDC& dc, const char far* str, int count, const TRect& rect, uint16 format);
    void          RetrieveHotTrackInfo();
    void          DrawButtonImage(TDC& dc);
    string        LoadInternalString(uint itemId);
    void          PaintBorder(TDC& dc);
    void          MouseEnter(uint modKeys, TPoint& point);
    void          MouseLeave(uint modKeys, TPoint& point);
    void          BeginPressed(TPoint& p);
    void          CancelPressed(TPoint& p);
    void          SetButtonState(TState newState);
    bool          IdleAction(long idleCount);
    void          SysColorChange();
    void          SetEnabled(bool enabled);

  public:
    // These allow to change the values at run-time.
    virtual void      SetMaxChars(uint maxchars);
    virtual uint      GetMaxChars() const;
    virtual void      SetTextRows(uint rows);
    virtual uint      GetTextRows() const;
    virtual void      SetDisplayType(JPDisplayType disp);
    virtual JPDisplayType GetDisplayType() const;
    virtual void      SetTextPosition(JPTextPos pos);
    virtual JPTextPos GetTextPosition() const;

    // Set the default values for this button.
    virtual void      SetDefaultDisplayType();
    virtual void      SetDefaultTextPosition();
    virtual void      SetDefaultTextRows();
    virtual void      SetDefaultMaxChars();

    //  Hot-Tracking feature.
    virtual  void     SetHotTrackColor(uint8 r, uint8 g, uint8 b);
    COLORREF          GetHotTrackColor();
    virtual void      SetHotTrack(bool track);
    virtual bool      GetHotTrack();

    // Methods for the text.
    virtual void      SetText(const char far* text);
    virtual const char far* GetText() const;

    // Methods for optional parameters.
    virtual void      SetOptionalParams(JPOptionalParams value, bool enable, bool update = true);
    virtual void      CheckOptionalParams();
    virtual bool      GetOptionalParams(JPOptionalParams value);
    virtual JPOptionalParams GetOptionalParamsValue();
    virtual void      InvertOptionalParams(JPOptionalParams value);
    virtual void      SetDefaultOptionalParameters();

    // derived methods.
    virtual void      GetCurrentBitmapSize(TSize& size);
    virtual void      GetDefaultBitmapSize(TSize& size);
    virtual TDib*     GetDefaultGlyphDib();
    virtual TDib*     GetImageDib();
    virtual void      GetDesiredSize(TSize& size);
//{{JPFlatButtonTextGadgetVIRTUAL_END}}
};    //{{JPFlatButtonTextGadget}}

#endif  // jpfltbtg_h sentry.

